@@ -16,8 +16,12 @@ from yarl import URL
 
 import aiohttp
 from aiohttp import BaseConnector, hdrs, payload
-from aiohttp.client_reqrep import (ClientRequest, ClientResponse, Fingerprint,
-                                   _merge_ssl_params)
+from aiohttp.client_reqrep import (
+    ClientRequest,
+    ClientResponse,
+    Fingerprint,
+    _merge_ssl_params,
+)
 from aiohttp.test_utils import make_mocked_coro
 
 
@@ -74,39 +78,39 @@ def conn(transport, protocol):
     )
 
 
-def test_method1(make_request):
+def test_method1(make_request) -> None:
     req = make_request('get', 'http://python.org/')
     assert req.method == 'GET'
 
 
-def test_method2(make_request):
+def test_method2(make_request) -> None:
     req = make_request('head', 'http://python.org/')
     assert req.method == 'HEAD'
 
 
-def test_method3(make_request):
+def test_method3(make_request) -> None:
     req = make_request('HEAD', 'http://python.org/')
     assert req.method == 'HEAD'
 
 
-def test_version_1_0(make_request):
+def test_version_1_0(make_request) -> None:
     req = make_request('get', 'http://python.org/', version='1.0')
     assert req.version == (1, 0)
 
 
-def test_version_default(make_request):
+def test_version_default(make_request) -> None:
     req = make_request('get', 'http://python.org/')
     assert req.version == (1, 1)
 
 
-def test_request_info(make_request):
+def test_request_info(make_request) -> None:
     req = make_request('get', 'http://python.org/')
     assert req.request_info == aiohttp.RequestInfo(URL('http://python.org/'),
                                                    'GET',
                                                    req.headers)
 
 
-def test_request_info_with_fragment(make_request):
+def test_request_info_with_fragment(make_request) -> None:
     req = make_request('get', 'http://python.org/#urlfragment')
     assert req.request_info == aiohttp.RequestInfo(
         URL('http://python.org/'),
@@ -114,18 +118,18 @@ def test_request_info_with_fragment(make_request):
         URL('http://python.org/#urlfragment'))
 
 
-def test_version_err(make_request):
+def test_version_err(make_request) -> None:
     with pytest.raises(ValueError):
         make_request('get', 'http://python.org/', version='1.c')
 
 
-def test_https_proxy(make_request):
+def test_https_proxy(make_request) -> None:
     with pytest.raises(ValueError):
         make_request(
             'get', 'http://python.org/', proxy=URL('https://proxy.org'))
 
 
-def test_keep_alive(make_request):
+def test_keep_alive(make_request) -> None:
     req = make_request('get', 'http://python.org/', version=(0, 9))
     assert not req.keep_alive()
 
@@ -144,123 +148,148 @@ def test_keep_alive(make_request):
     assert not req.keep_alive()
 
 
-def test_host_port_default_http(make_request):
+def test_host_port_default_http(make_request) -> None:
     req = make_request('get', 'http://python.org/')
     assert req.host == 'python.org'
     assert req.port == 80
     assert not req.ssl
 
 
-def test_host_port_default_https(make_request):
+def test_host_port_default_https(make_request) -> None:
     req = make_request('get', 'https://python.org/')
     assert req.host == 'python.org'
     assert req.port == 443
     assert req.is_ssl()
 
 
-def test_host_port_nondefault_http(make_request):
+def test_host_port_nondefault_http(make_request) -> None:
     req = make_request('get', 'http://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
     assert not req.is_ssl()
 
 
-def test_host_port_nondefault_https(make_request):
+def test_host_port_nondefault_https(make_request) -> None:
     req = make_request('get', 'https://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
     assert req.is_ssl()
 
 
-def test_host_port_default_ws(make_request):
+def test_host_port_default_ws(make_request) -> None:
     req = make_request('get', 'ws://python.org/')
     assert req.host == 'python.org'
     assert req.port == 80
     assert not req.is_ssl()
 
 
-def test_host_port_default_wss(make_request):
+def test_host_port_default_wss(make_request) -> None:
     req = make_request('get', 'wss://python.org/')
     assert req.host == 'python.org'
     assert req.port == 443
     assert req.is_ssl()
 
 
-def test_host_port_nondefault_ws(make_request):
+def test_host_port_nondefault_ws(make_request) -> None:
     req = make_request('get', 'ws://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
     assert not req.is_ssl()
 
 
-def test_host_port_nondefault_wss(make_request):
+def test_host_port_nondefault_wss(make_request) -> None:
     req = make_request('get', 'wss://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
     assert req.is_ssl()
 
 
-def test_host_port_err(make_request):
+def test_host_port_err(make_request) -> None:
     with pytest.raises(ValueError):
         make_request('get', 'http://python.org:123e/')
 
 
-def test_hostname_err(make_request):
+def test_hostname_err(make_request) -> None:
     with pytest.raises(ValueError):
         make_request('get', 'http://:8080/')
 
 
-def test_host_header_host_without_port(make_request):
+def test_host_header_host_first(make_request) -> None:
+    req = make_request('get', 'http://python.org/')
+    assert list(req.headers)[0] == 'Host'
+
+
+def test_host_header_host_without_port(make_request) -> None:
     req = make_request('get', 'http://python.org/')
     assert req.headers['HOST'] == 'python.org'
 
 
-def test_host_header_host_with_default_port(make_request):
+def test_host_header_host_with_default_port(make_request) -> None:
     req = make_request('get', 'http://python.org:80/')
     assert req.headers['HOST'] == 'python.org'
 
 
-def test_host_header_host_with_nondefault_port(make_request):
+def test_host_header_host_with_nondefault_port(make_request) -> None:
     req = make_request('get', 'http://python.org:99/')
     assert req.headers['HOST'] == 'python.org:99'
 
 
-def test_host_header_host_idna_encode(make_request):
+def test_host_header_host_idna_encode(make_request) -> None:
     req = make_request('get', 'http://xn--9caa.com')
     assert req.headers['HOST'] == 'xn--9caa.com'
 
 
-def test_host_header_host_unicode(make_request):
+def test_host_header_host_unicode(make_request) -> None:
     req = make_request('get', 'http://éé.com')
     assert req.headers['HOST'] == 'xn--9caa.com'
 
 
-def test_host_header_explicit_host(make_request):
+def test_host_header_explicit_host(make_request) -> None:
     req = make_request('get', 'http://python.org/',
                        headers={'host': 'example.com'})
     assert req.headers['HOST'] == 'example.com'
 
 
-def test_host_header_explicit_host_with_port(make_request):
+def test_host_header_explicit_host_with_port(make_request) -> None:
     req = make_request('get', 'http://python.org/',
                        headers={'host': 'example.com:99'})
     assert req.headers['HOST'] == 'example.com:99'
 
 
-def test_default_loop(loop):
+def test_host_header_ipv4(make_request) -> None:
+    req = make_request('get', 'http://127.0.0.2')
+    assert req.headers['HOST'] == '127.0.0.2'
+
+
+def test_host_header_ipv6(make_request) -> None:
+    req = make_request('get', 'http://[::2]')
+    assert req.headers['HOST'] == '[::2]'
+
+
+def test_host_header_ipv4_with_port(make_request) -> None:
+    req = make_request('get', 'http://127.0.0.2:99')
+    assert req.headers['HOST'] == '127.0.0.2:99'
+
+
+def test_host_header_ipv6_with_port(make_request) -> None:
+    req = make_request('get', 'http://[::2]:99')
+    assert req.headers['HOST'] == '[::2]:99'
+
+
+def test_default_loop(loop) -> None:
     asyncio.set_event_loop(loop)
     req = ClientRequest('get', URL('http://python.org/'))
     assert req.loop is loop
 
 
-def test_default_headers_useragent(make_request):
+def test_default_headers_useragent(make_request) -> None:
     req = make_request('get', 'http://python.org/')
 
     assert 'SERVER' not in req.headers
     assert 'USER-AGENT' in req.headers
 
 
-def test_default_headers_useragent_custom(make_request):
+def test_default_headers_useragent_custom(make_request) -> None:
     req = make_request('get', 'http://python.org/',
                        headers={'user-agent': 'my custom agent'})
 
@@ -268,14 +297,14 @@ def test_default_headers_useragent_custom(make_request):
     assert 'my custom agent' == req.headers['User-Agent']
 
 
-def test_skip_default_useragent_header(make_request):
+def test_skip_default_useragent_header(make_request) -> None:
     req = make_request('get', 'http://python.org/',
                        skip_auto_headers=set([istr('user-agent')]))
 
     assert 'User-Agent' not in req.headers
 
 
-def test_headers(make_request):
+def test_headers(make_request) -> None:
     req = make_request('post', 'http://python.org/',
                        headers={'Content-Type': 'text/plain'})
 
@@ -284,85 +313,85 @@ def test_headers(make_request):
     assert req.headers['ACCEPT-ENCODING'] == 'gzip, deflate'
 
 
-def test_headers_list(make_request):
+def test_headers_list(make_request) -> None:
     req = make_request('post', 'http://python.org/',
                        headers=[('Content-Type', 'text/plain')])
     assert 'CONTENT-TYPE' in req.headers
     assert req.headers['CONTENT-TYPE'] == 'text/plain'
 
 
-def test_headers_default(make_request):
+def test_headers_default(make_request) -> None:
     req = make_request('get', 'http://python.org/',
                        headers={'ACCEPT-ENCODING': 'deflate'})
     assert req.headers['ACCEPT-ENCODING'] == 'deflate'
 
 
-def test_invalid_url(make_request):
+def test_invalid_url(make_request) -> None:
     with pytest.raises(aiohttp.InvalidURL):
         make_request('get', 'hiwpefhipowhefopw')
 
 
-def test_no_path(make_request):
+def test_no_path(make_request) -> None:
     req = make_request('get', 'http://python.org')
     assert '/' == req.url.path
 
 
-def test_ipv6_default_http_port(make_request):
+def test_ipv6_default_http_port(make_request) -> None:
     req = make_request('get', 'http://[2001:db8::1]/')
     assert req.host == '2001:db8::1'
     assert req.port == 80
     assert not req.ssl
 
 
-def test_ipv6_default_https_port(make_request):
+def test_ipv6_default_https_port(make_request) -> None:
     req = make_request('get', 'https://[2001:db8::1]/')
     assert req.host == '2001:db8::1'
     assert req.port == 443
     assert req.is_ssl()
 
 
-def test_ipv6_nondefault_http_port(make_request):
+def test_ipv6_nondefault_http_port(make_request) -> None:
     req = make_request('get', 'http://[2001:db8::1]:960/')
     assert req.host == '2001:db8::1'
     assert req.port == 960
     assert not req.is_ssl()
 
 
-def test_ipv6_nondefault_https_port(make_request):
+def test_ipv6_nondefault_https_port(make_request) -> None:
     req = make_request('get', 'https://[2001:db8::1]:960/')
     assert req.host == '2001:db8::1'
     assert req.port == 960
     assert req.is_ssl()
 
 
-def test_basic_auth(make_request):
+def test_basic_auth(make_request) -> None:
     req = make_request('get', 'http://python.org',
                        auth=aiohttp.BasicAuth('nkim', '1234'))
     assert 'AUTHORIZATION' in req.headers
     assert 'Basic bmtpbToxMjM0' == req.headers['AUTHORIZATION']
 
 
-def test_basic_auth_utf8(make_request):
+def test_basic_auth_utf8(make_request) -> None:
     req = make_request('get', 'http://python.org',
                        auth=aiohttp.BasicAuth('nkim', 'секрет', 'utf-8'))
     assert 'AUTHORIZATION' in req.headers
     assert 'Basic bmtpbTrRgdC10LrRgNC10YI=' == req.headers['AUTHORIZATION']
 
 
-def test_basic_auth_tuple_forbidden(make_request):
+def test_basic_auth_tuple_forbidden(make_request) -> None:
     with pytest.raises(TypeError):
         make_request('get', 'http://python.org',
                      auth=('nkim', '1234'))
 
 
-def test_basic_auth_from_url(make_request):
+def test_basic_auth_from_url(make_request) -> None:
     req = make_request('get', 'http://nkim:1234@python.org')
     assert 'AUTHORIZATION' in req.headers
     assert 'Basic bmtpbToxMjM0' == req.headers['AUTHORIZATION']
     assert 'python.org' == req.host
 
 
-def test_basic_auth_from_url_overriden(make_request):
+def test_basic_auth_from_url_overridden(make_request) -> None:
     req = make_request('get', 'http://garbage@python.org',
                        auth=aiohttp.BasicAuth('nkim', '1234'))
     assert 'AUTHORIZATION' in req.headers
@@ -370,49 +399,49 @@ def test_basic_auth_from_url_overriden(make_request):
     assert 'python.org' == req.host
 
 
-def test_path_is_not_double_encoded1(make_request):
+def test_path_is_not_double_encoded1(make_request) -> None:
     req = make_request('get', "http://0.0.0.0/get/test case")
     assert req.url.raw_path == "/get/test%20case"
 
 
-def test_path_is_not_double_encoded2(make_request):
+def test_path_is_not_double_encoded2(make_request) -> None:
     req = make_request('get', "http://0.0.0.0/get/test%2fcase")
     assert req.url.raw_path == "/get/test%2Fcase"
 
 
-def test_path_is_not_double_encoded3(make_request):
+def test_path_is_not_double_encoded3(make_request) -> None:
     req = make_request('get', "http://0.0.0.0/get/test%20case")
     assert req.url.raw_path == "/get/test%20case"
 
 
-def test_path_safe_chars_preserved(make_request):
+def test_path_safe_chars_preserved(make_request) -> None:
     req = make_request('get', "http://0.0.0.0/get/:=+/%2B/")
     assert req.url.path == "/get/:=+/+/"
 
 
-def test_params_are_added_before_fragment1(make_request):
+def test_params_are_added_before_fragment1(make_request) -> None:
     req = make_request('GET', "http://example.com/path#fragment",
                        params={"a": "b"})
     assert str(req.url) == "http://example.com/path?a=b"
 
 
-def test_params_are_added_before_fragment2(make_request):
+def test_params_are_added_before_fragment2(make_request) -> None:
     req = make_request('GET', "http://example.com/path?key=value#fragment",
                        params={"a": "b"})
     assert str(req.url) == "http://example.com/path?key=value&a=b"
 
 
-def test_path_not_contain_fragment1(make_request):
+def test_path_not_contain_fragment1(make_request) -> None:
     req = make_request('GET', "http://example.com/path#fragment")
     assert req.url.path == "/path"
 
 
-def test_path_not_contain_fragment2(make_request):
+def test_path_not_contain_fragment2(make_request) -> None:
     req = make_request('GET', "http://example.com/path?key=value#fragment")
     assert str(req.url) == "http://example.com/path?key=value"
 
 
-def test_cookies(make_request):
+def test_cookies(make_request) -> None:
     req = make_request('get', 'http://test.com/path',
                        cookies={'cookie1': 'val1'})
 
@@ -420,7 +449,7 @@ def test_cookies(make_request):
     assert 'cookie1=val1' == req.headers['COOKIE']
 
 
-def test_cookies_merge_with_headers(make_request):
+def test_cookies_merge_with_headers(make_request) -> None:
     req = make_request('get', 'http://test.com/path',
                        headers={'cookie': 'cookie1=val1'},
                        cookies={'cookie2': 'val2'})
@@ -428,25 +457,25 @@ def test_cookies_merge_with_headers(make_request):
     assert 'cookie1=val1; cookie2=val2' == req.headers['COOKIE']
 
 
-def test_unicode_get1(make_request):
+def test_unicode_get1(make_request) -> None:
     req = make_request('get', 'http://python.org',
                        params={'foo': 'f\xf8\xf8'})
     assert 'http://python.org/?foo=f%C3%B8%C3%B8' == str(req.url)
 
 
-def test_unicode_get2(make_request):
+def test_unicode_get2(make_request) -> None:
     req = make_request('', 'http://python.org',
                        params={'f\xf8\xf8': 'f\xf8\xf8'})
 
     assert 'http://python.org/?f%C3%B8%C3%B8=f%C3%B8%C3%B8' == str(req.url)
 
 
-def test_unicode_get3(make_request):
+def test_unicode_get3(make_request) -> None:
     req = make_request('', 'http://python.org', params={'foo': 'foo'})
     assert 'http://python.org/?foo=foo' == str(req.url)
 
 
-def test_unicode_get4(make_request):
+def test_unicode_get4(make_request) -> None:
     def join(*suffix):
         return urllib.parse.urljoin('http://python.org/', '/'.join(suffix))
 
@@ -454,7 +483,7 @@ def test_unicode_get4(make_request):
     assert 'http://python.org/%C3%B8?foo=foo' == str(req.url)
 
 
-def test_query_multivalued_param(make_request):
+def test_query_multivalued_param(make_request) -> None:
     for meth in ClientRequest.ALL_METHODS:
         req = make_request(
             meth, 'http://python.org',
@@ -463,38 +492,38 @@ def test_query_multivalued_param(make_request):
         assert str(req.url) == 'http://python.org/?test=foo&test=baz'
 
 
-def test_query_str_param(make_request):
+def test_query_str_param(make_request) -> None:
     for meth in ClientRequest.ALL_METHODS:
         req = make_request(meth, 'http://python.org', params='test=foo')
         assert str(req.url) == 'http://python.org/?test=foo'
 
 
-def test_query_bytes_param_raises(make_request):
+def test_query_bytes_param_raises(make_request) -> None:
     for meth in ClientRequest.ALL_METHODS:
         with pytest.raises(TypeError):
             make_request(meth, 'http://python.org', params=b'test=foo')
 
 
-def test_query_str_param_is_not_encoded(make_request):
+def test_query_str_param_is_not_encoded(make_request) -> None:
     for meth in ClientRequest.ALL_METHODS:
         req = make_request(meth, 'http://python.org', params='test=f+oo')
         assert str(req.url) == 'http://python.org/?test=f+oo'
 
 
-def test_params_update_path_and_url(make_request):
+def test_params_update_path_and_url(make_request) -> None:
     req = make_request('get', 'http://python.org',
                        params=(('test', 'foo'), ('test', 'baz')))
     assert str(req.url) == 'http://python.org/?test=foo&test=baz'
 
 
-def test_params_empty_path_and_url(make_request):
+def test_params_empty_path_and_url(make_request) -> None:
     req_empty = make_request('get', 'http://python.org', params={})
     assert str(req_empty.url) == 'http://python.org'
     req_none = make_request('get', 'http://python.org')
     assert str(req_none.url) == 'http://python.org'
 
 
-def test_gen_netloc_all(make_request):
+def test_gen_netloc_all(make_request) -> None:
     req = make_request('get',
                        'https://aiohttp:pwpwpw@' +
                        '12345678901234567890123456789' +
@@ -503,7 +532,7 @@ def test_gen_netloc_all(make_request):
         '012345678901234567890:8080'
 
 
-def test_gen_netloc_no_port(make_request):
+def test_gen_netloc_no_port(make_request) -> None:
     req = make_request('get',
                        'https://aiohttp:pwpwpw@' +
                        '12345678901234567890123456789' +
@@ -512,7 +541,7 @@ def test_gen_netloc_no_port(make_request):
         '012345678901234567890'
 
 
-async def test_connection_header(loop, conn):
+async def test_connection_header(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     req.keep_alive = mock.Mock()
     req.headers.clear()
@@ -535,7 +564,7 @@ async def test_connection_header(loop, conn):
     assert req.headers.get('CONNECTION') == 'close'
 
 
-async def test_no_content_length(loop, conn):
+async def test_no_content_length(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     resp = await req.send(conn)
     assert req.headers.get('CONTENT-LENGTH') is None
@@ -543,7 +572,7 @@ async def test_no_content_length(loop, conn):
     resp.close()
 
 
-async def test_no_content_length_head(loop, conn):
+async def test_no_content_length_head(loop, conn) -> None:
     req = ClientRequest('head', URL('http://python.org'), loop=loop)
     resp = await req.send(conn)
     assert req.headers.get('CONTENT-LENGTH') is None
@@ -551,14 +580,14 @@ async def test_no_content_length_head(loop, conn):
     resp.close()
 
 
-async def test_content_type_auto_header_get(loop, conn):
+async def test_content_type_auto_header_get(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     resp = await req.send(conn)
     assert 'CONTENT-TYPE' not in req.headers
     resp.close()
 
 
-async def test_content_type_auto_header_form(loop, conn):
+async def test_content_type_auto_header_form(loop, conn) -> None:
     req = ClientRequest('post', URL('http://python.org'),
                         data={'hey': 'you'}, loop=loop)
     resp = await req.send(conn)
@@ -567,7 +596,7 @@ async def test_content_type_auto_header_form(loop, conn):
     resp.close()
 
 
-async def test_content_type_auto_header_bytes(loop, conn):
+async def test_content_type_auto_header_bytes(loop, conn) -> None:
     req = ClientRequest('post', URL('http://python.org'), data=b'hey you',
                         loop=loop)
     resp = await req.send(conn)
@@ -575,7 +604,7 @@ async def test_content_type_auto_header_bytes(loop, conn):
     resp.close()
 
 
-async def test_content_type_skip_auto_header_bytes(loop, conn):
+async def test_content_type_skip_auto_header_bytes(loop, conn) -> None:
     req = ClientRequest('post', URL('http://python.org'), data=b'hey you',
                         skip_auto_headers={'Content-Type'},
                         loop=loop)
@@ -584,7 +613,7 @@ async def test_content_type_skip_auto_header_bytes(loop, conn):
     resp.close()
 
 
-async def test_content_type_skip_auto_header_form(loop, conn):
+async def test_content_type_skip_auto_header_form(loop, conn) -> None:
     req = ClientRequest('post', URL('http://python.org'),
                         data={'hey': 'you'}, loop=loop,
                         skip_auto_headers={'Content-Type'})
@@ -593,7 +622,8 @@ async def test_content_type_skip_auto_header_form(loop, conn):
     resp.close()
 
 
-async def test_content_type_auto_header_content_length_no_skip(loop, conn):
+async def test_content_type_auto_header_content_length_no_skip(loop,
+                                                               conn) -> None:
     req = ClientRequest('post', URL('http://python.org'),
                         data=io.BytesIO(b'hey'),
                         skip_auto_headers={'Content-Length'},
@@ -603,7 +633,7 @@ async def test_content_type_auto_header_content_length_no_skip(loop, conn):
     resp.close()
 
 
-async def test_urlencoded_formdata_charset(loop, conn):
+async def test_urlencoded_formdata_charset(loop, conn) -> None:
     req = ClientRequest(
         'post', URL('http://python.org'),
         data=aiohttp.FormData({'hey': 'you'}, charset='koi8-r'), loop=loop)
@@ -612,7 +642,7 @@ async def test_urlencoded_formdata_charset(loop, conn):
         req.headers.get('CONTENT-TYPE')
 
 
-async def test_post_data(loop, conn):
+async def test_post_data(loop, conn) -> None:
     for meth in ClientRequest.POST_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'),
@@ -626,7 +656,7 @@ async def test_post_data(loop, conn):
         resp.close()
 
 
-async def test_pass_falsy_data(loop):
+async def test_pass_falsy_data(loop) -> None:
     with mock.patch(
             'aiohttp.client_reqrep.ClientRequest.update_body_from_data'):
         req = ClientRequest(
@@ -636,7 +666,7 @@ async def test_pass_falsy_data(loop):
     await req.close()
 
 
-async def test_pass_falsy_data_file(loop, tmpdir):
+async def test_pass_falsy_data_file(loop, tmpdir) -> None:
     testfile = tmpdir.join('tmpfile').open('w+b')
     testfile.write(b'data')
     testfile.seek(0)
@@ -651,7 +681,7 @@ async def test_pass_falsy_data_file(loop, tmpdir):
 
 
 # Elasticsearch API requires to send request body with GET-requests
-async def test_get_with_data(loop):
+async def test_get_with_data(loop) -> None:
     for meth in ClientRequest.GET_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'), data={'life': '42'},
@@ -661,7 +691,7 @@ async def test_get_with_data(loop):
         await req.close()
 
 
-async def test_bytes_data(loop, conn):
+async def test_bytes_data(loop, conn) -> None:
     for meth in ClientRequest.POST_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'),
@@ -675,7 +705,7 @@ async def test_bytes_data(loop, conn):
         resp.close()
 
 
-async def test_content_encoding(loop, conn):
+async def test_content_encoding(loop, conn) -> None:
     req = ClientRequest('post', URL('http://python.org/'), data='foo',
                         compress='deflate', loop=loop)
     with mock.patch('aiohttp.client_reqrep.StreamWriter') as m_writer:
@@ -689,7 +719,8 @@ async def test_content_encoding(loop, conn):
     resp.close()
 
 
-async def test_content_encoding_dont_set_headers_if_no_body(loop, conn):
+async def test_content_encoding_dont_set_headers_if_no_body(loop,
+                                                            conn) -> None:
     req = ClientRequest('post', URL('http://python.org/'),
                         compress='deflate', loop=loop)
     with mock.patch('aiohttp.client_reqrep.http'):
@@ -700,7 +731,7 @@ async def test_content_encoding_dont_set_headers_if_no_body(loop, conn):
     resp.close()
 
 
-async def test_content_encoding_header(loop, conn):
+async def test_content_encoding_header(loop, conn) -> None:
     req = ClientRequest(
         'post', URL('http://python.org/'), data='foo',
         headers={'Content-Encoding': 'deflate'}, loop=loop)
@@ -714,14 +745,14 @@ async def test_content_encoding_header(loop, conn):
     resp.close()
 
 
-async def test_compress_and_content_encoding(loop, conn):
+async def test_compress_and_content_encoding(loop, conn) -> None:
     with pytest.raises(ValueError):
         ClientRequest('post', URL('http://python.org/'), data='foo',
                       headers={'content-encoding': 'deflate'},
                       compress='deflate', loop=loop)
 
 
-async def test_chunked(loop, conn):
+async def test_chunked(loop, conn) -> None:
     req = ClientRequest(
         'post', URL('http://python.org/'),
         headers={'TRANSFER-ENCODING': 'gzip'}, loop=loop)
@@ -731,7 +762,7 @@ async def test_chunked(loop, conn):
     resp.close()
 
 
-async def test_chunked2(loop, conn):
+async def test_chunked2(loop, conn) -> None:
     req = ClientRequest(
         'post', URL('http://python.org/'),
         headers={'Transfer-encoding': 'chunked'}, loop=loop)
@@ -741,7 +772,7 @@ async def test_chunked2(loop, conn):
     resp.close()
 
 
-async def test_chunked_explicit(loop, conn):
+async def test_chunked_explicit(loop, conn) -> None:
     req = ClientRequest(
         'post', URL('http://python.org/'), chunked=True, loop=loop)
     with mock.patch('aiohttp.client_reqrep.StreamWriter') as m_writer:
@@ -754,23 +785,23 @@ async def test_chunked_explicit(loop, conn):
     resp.close()
 
 
-async def test_chunked_length(loop, conn):
+async def test_chunked_length(loop, conn) -> None:
     with pytest.raises(ValueError):
         ClientRequest(
             'post', URL('http://python.org/'),
             headers={'CONTENT-LENGTH': '1000'}, chunked=True, loop=loop)
 
 
-async def test_chunked_transfer_encoding(loop, conn):
+async def test_chunked_transfer_encoding(loop, conn) -> None:
     with pytest.raises(ValueError):
         ClientRequest(
             'post', URL('http://python.org/'),
             headers={'TRANSFER-ENCODING': 'chunked'}, chunked=True, loop=loop)
 
 
-async def test_file_upload_not_chunked(loop):
+async def test_file_upload_not_chunked(loop) -> None:
     here = os.path.dirname(__file__)
-    fname = os.path.join(here, 'sample.key')
+    fname = os.path.join(here, 'aiohttp.png')
     with open(fname, 'rb') as f:
         req = ClientRequest(
             'post', URL('http://python.org/'),
@@ -781,7 +812,7 @@ async def test_file_upload_not_chunked(loop):
         await req.close()
 
 
-async def test_precompressed_data_stays_intact(loop):
+async def test_precompressed_data_stays_intact(loop) -> None:
     data = zlib.compress(b'foobar')
     req = ClientRequest(
         'post', URL('http://python.org/'),
@@ -795,9 +826,9 @@ async def test_precompressed_data_stays_intact(loop):
     await req.close()
 
 
-async def test_file_upload_not_chunked_seek(loop):
+async def test_file_upload_not_chunked_seek(loop) -> None:
     here = os.path.dirname(__file__)
-    fname = os.path.join(here, 'sample.key')
+    fname = os.path.join(here, 'aiohttp.png')
     with open(fname, 'rb') as f:
         f.seek(100)
         req = ClientRequest(
@@ -809,9 +840,9 @@ async def test_file_upload_not_chunked_seek(loop):
         await req.close()
 
 
-async def test_file_upload_force_chunked(loop):
+async def test_file_upload_force_chunked(loop) -> None:
     here = os.path.dirname(__file__)
-    fname = os.path.join(here, 'sample.key')
+    fname = os.path.join(here, 'aiohttp.png')
     with open(fname, 'rb') as f:
         req = ClientRequest(
             'post', URL('http://python.org/'),
@@ -823,7 +854,7 @@ async def test_file_upload_force_chunked(loop):
         await req.close()
 
 
-async def test_expect100(loop, conn):
+async def test_expect100(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org/'),
                         expect100=True, loop=loop)
     resp = await req.send(conn)
@@ -833,7 +864,7 @@ async def test_expect100(loop, conn):
     resp.close()
 
 
-async def test_expect_100_continue_header(loop, conn):
+async def test_expect_100_continue_header(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org/'),
                         headers={'expect': '100-continue'}, loop=loop)
     resp = await req.send(conn)
@@ -843,7 +874,7 @@ async def test_expect_100_continue_header(loop, conn):
     resp.close()
 
 
-async def test_data_stream(loop, buf, conn):
+async def test_data_stream(loop, buf, conn) -> None:
     @async_generator
     async def gen():
         await yield_(b'binary data')
@@ -863,7 +894,7 @@ async def test_data_stream(loop, buf, conn):
     await req.close()
 
 
-async def test_data_stream_deprecated(loop, buf, conn):
+async def test_data_stream_deprecated(loop, buf, conn) -> None:
     with pytest.warns(DeprecationWarning):
         @aiohttp.streamer
         async def gen(writer):
@@ -884,7 +915,7 @@ async def test_data_stream_deprecated(loop, buf, conn):
     await req.close()
 
 
-async def test_data_file(loop, buf, conn):
+async def test_data_file(loop, buf, conn) -> None:
     req = ClientRequest(
         'POST', URL('http://python.org/'),
         data=io.BufferedReader(io.BytesIO(b'*' * 2)),
@@ -902,7 +933,7 @@ async def test_data_file(loop, buf, conn):
     await req.close()
 
 
-async def test_data_stream_exc(loop, conn):
+async def test_data_stream_exc(loop, conn) -> None:
     fut = loop.create_future()
 
     @async_generator
@@ -928,7 +959,7 @@ async def test_data_stream_exc(loop, conn):
     await req.close()
 
 
-async def test_data_stream_exc_deprecated(loop, conn):
+async def test_data_stream_exc_deprecated(loop, conn) -> None:
     fut = loop.create_future()
 
     with pytest.warns(DeprecationWarning):
@@ -955,7 +986,7 @@ async def test_data_stream_exc_deprecated(loop, conn):
     await req.close()
 
 
-async def test_data_stream_exc_chain(loop, conn):
+async def test_data_stream_exc_chain(loop, conn) -> None:
     fut = loop.create_future()
 
     @async_generator
@@ -984,7 +1015,7 @@ async def test_data_stream_exc_chain(loop, conn):
     await req.close()
 
 
-async def test_data_stream_exc_chain_deprecated(loop, conn):
+async def test_data_stream_exc_chain_deprecated(loop, conn) -> None:
     fut = loop.create_future()
 
     with pytest.warns(DeprecationWarning):
@@ -1014,7 +1045,7 @@ async def test_data_stream_exc_chain_deprecated(loop, conn):
     await req.close()
 
 
-async def test_data_stream_continue(loop, buf, conn):
+async def test_data_stream_continue(loop, buf, conn) -> None:
     @async_generator
     async def gen():
         await yield_(b'binary data')
@@ -1039,7 +1070,7 @@ async def test_data_stream_continue(loop, buf, conn):
     resp.close()
 
 
-async def test_data_stream_continue_deprecated(loop, buf, conn):
+async def test_data_stream_continue_deprecated(loop, buf, conn) -> None:
     with pytest.warns(DeprecationWarning):
         @aiohttp.streamer
         async def gen(writer):
@@ -1066,7 +1097,7 @@ async def test_data_stream_continue_deprecated(loop, buf, conn):
     resp.close()
 
 
-async def test_data_continue(loop, buf, conn):
+async def test_data_continue(loop, buf, conn) -> None:
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=b'data',
         expect100=True, loop=loop)
@@ -1085,7 +1116,7 @@ async def test_data_continue(loop, buf, conn):
     resp.close()
 
 
-async def test_close(loop, buf, conn):
+async def test_close(loop, buf, conn) -> None:
     @async_generator
     async def gen():
         await asyncio.sleep(0.00001)
@@ -1100,7 +1131,7 @@ async def test_close(loop, buf, conn):
     resp.close()
 
 
-async def test_close_deprecated(loop, buf, conn):
+async def test_close_deprecated(loop, buf, conn) -> None:
     with pytest.warns(DeprecationWarning):
         @aiohttp.streamer
         async def gen(writer):
@@ -1116,7 +1147,7 @@ async def test_close_deprecated(loop, buf, conn):
     resp.close()
 
 
-async def test_custom_response_class(loop, conn):
+async def test_custom_response_class(loop, conn) -> None:
     class CustomResponse(ClientResponse):
         def read(self, decode=False):
             return 'customized!'
@@ -1130,7 +1161,7 @@ async def test_custom_response_class(loop, conn):
     resp.close()
 
 
-async def test_oserror_on_write_bytes(loop, conn):
+async def test_oserror_on_write_bytes(loop, conn) -> None:
     req = ClientRequest(
         'POST', URL('http://python.org/'), loop=loop)
 
@@ -1144,7 +1175,7 @@ async def test_oserror_on_write_bytes(loop, conn):
     assert isinstance(exc, aiohttp.ClientOSError)
 
 
-async def test_terminate(loop, conn):
+async def test_terminate(loop, conn) -> None:
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     resp = await req.send(conn)
     assert req._writer is not None
@@ -1156,7 +1187,7 @@ async def test_terminate(loop, conn):
     resp.close()
 
 
-def test_terminate_with_closed_loop(loop, conn):
+def test_terminate_with_closed_loop(loop, conn) -> None:
     req = resp = writer = None
 
     async def go():
@@ -1177,7 +1208,7 @@ def test_terminate_with_closed_loop(loop, conn):
     resp.close()
 
 
-def test_terminate_without_writer(loop):
+def test_terminate_without_writer(loop) -> None:
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     assert req._writer is None
 
@@ -1185,7 +1216,7 @@ def test_terminate_without_writer(loop):
     assert req._writer is None
 
 
-async def test_custom_req_rep(loop):
+async def test_custom_req_rep(loop) -> None:
     conn = None
 
     class CustomResponse(ClientResponse):
@@ -1239,23 +1270,23 @@ async def test_custom_req_rep(loop):
     conn.close()
 
 
-def test_verify_ssl_false_with_ssl_context(loop):
+def test_verify_ssl_false_with_ssl_context(loop, ssl_ctx) -> None:
     with pytest.warns(DeprecationWarning):
         with pytest.raises(ValueError):
             _merge_ssl_params(None, verify_ssl=False,
-                              ssl_context=mock.Mock(), fingerprint=None)
+                              ssl_context=ssl_ctx, fingerprint=None)
 
 
-def test_bad_fingerprint(loop):
+def test_bad_fingerprint(loop) -> None:
     with pytest.raises(ValueError):
         Fingerprint(b'invalid')
 
 
-def test_insecure_fingerprint_md5(loop):
+def test_insecure_fingerprint_md5(loop) -> None:
     with pytest.raises(ValueError):
         Fingerprint(hashlib.md5(b"foo").digest())
 
 
-def test_insecure_fingerprint_sha1(loop):
+def test_insecure_fingerprint_sha1(loop) -> None:
     with pytest.raises(ValueError):
         Fingerprint(hashlib.sha1(b"foo").digest())
