@@ -3,10 +3,10 @@
 import asyncio
 import gc
 import hashlib
-import os.path
 import platform
 import socket
 import ssl
+import sys
 import uuid
 from collections import deque
 from unittest import mock
@@ -19,6 +19,7 @@ from aiohttp import client, web
 from aiohttp.client import ClientRequest, ClientTimeout
 from aiohttp.client_reqrep import ConnectionKey
 from aiohttp.connector import Connection, _DNSCacheTable
+from aiohttp.helpers import PY_37
 from aiohttp.test_utils import make_mocked_coro, unused_port
 from aiohttp.tracing import Trace
 
@@ -64,7 +65,7 @@ def unix_server(loop, unix_sockname):
         loop.run_until_complete(runner.cleanup())
 
 
-def test_connection_del(loop):
+def test_connection_del(loop) -> None:
     connector = mock.Mock()
     key = mock.Mock()
     protocol = mock.Mock()
@@ -89,7 +90,7 @@ def test_connection_del(loop):
     exc_handler.assert_called_with(loop, msg)
 
 
-def test_connection_del_loop_debug(loop):
+def test_connection_del_loop_debug(loop) -> None:
     connector = mock.Mock()
     key = mock.Mock()
     protocol = mock.Mock()
@@ -110,7 +111,7 @@ def test_connection_del_loop_debug(loop):
     exc_handler.assert_called_with(loop, msg)
 
 
-def test_connection_del_loop_closed(loop):
+def test_connection_del_loop_closed(loop) -> None:
     connector = mock.Mock()
     key = mock.Mock()
     protocol = mock.Mock()
@@ -128,19 +129,8 @@ def test_connection_del_loop_closed(loop):
     assert not exc_handler.called
 
 
-def test_connection_detach(loop):
-    connector = mock.Mock()
-    key = mock.Mock()
-    protocol = mock.Mock()
-    conn = Connection(connector, key, protocol, loop=loop)
-    conn._notify_release = mock.Mock()
-    conn.detach()
-    assert conn._notify_release.called
-    connector._release_acquired.assert_called_with(protocol)
-
-
-def test_del(loop):
-    conn = aiohttp.BaseConnector(loop=loop)
+async def test_del(loop) -> None:
+    conn = aiohttp.BaseConnector()
     proto = mock.Mock(should_close=False)
     conn._release('a', proto)
     conns_impl = conn._conns
@@ -163,7 +153,7 @@ def test_del(loop):
 
 
 @pytest.mark.xfail
-async def test_del_with_scheduled_cleanup(loop):
+async def test_del_with_scheduled_cleanup(loop) -> None:
     loop.set_debug(True)
     conn = aiohttp.BaseConnector(loop=loop, keepalive_timeout=0.01)
     transp = mock.Mock()
@@ -189,8 +179,12 @@ async def test_del_with_scheduled_cleanup(loop):
     exc_handler.assert_called_with(loop, msg)
 
 
-def test_del_with_closed_loop(loop):
-    conn = aiohttp.BaseConnector(loop=loop)
+@pytest.mark.skipif(sys.implementation.name != 'cpython',
+                    reason="CPython GC is required for the test")
+def test_del_with_closed_loop(loop) -> None:
+    async def make_conn():
+        return aiohttp.BaseConnector()
+    conn = loop.run_until_complete(make_conn())
     transp = mock.Mock()
     conn._conns['a'] = [(transp, 123)]
 
@@ -208,7 +202,7 @@ def test_del_with_closed_loop(loop):
     assert exc_handler.called
 
 
-def test_del_empty_conector(loop):
+async def test_del_empty_connector(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
 
     exc_handler = mock.Mock()
@@ -219,30 +213,32 @@ def test_del_empty_conector(loop):
     assert not exc_handler.called
 
 
-async def test_create_conn(loop):
+async def test_create_conn(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     with pytest.raises(NotImplementedError):
         await conn._create_connection(object(), [], object())
 
 
-def test_context_manager(loop):
+async def test_context_manager(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
-    conn.close = mock.Mock()
 
-    with conn as c:
+    with pytest.warns(DeprecationWarning):
+        with conn as c:
+            assert conn is c
+
+    assert conn.closed
+
+
+async def test_async_context_manager(loop) -> None:
+    conn = aiohttp.BaseConnector(loop=loop)
+
+    async with conn as c:
         assert conn is c
 
-    assert conn.close.called
+    assert conn.closed
 
 
-def test_ctor_loop():
-    with mock.patch('aiohttp.connector.asyncio') as m_asyncio:
-        session = aiohttp.BaseConnector()
-
-    assert session._loop is m_asyncio.get_event_loop.return_value
-
-
-def test_close(loop):
+async def test_close(loop) -> None:
     proto = mock.Mock()
 
     conn = aiohttp.BaseConnector(loop=loop)
@@ -255,7 +251,7 @@ def test_close(loop):
     assert conn.closed
 
 
-def test_get(loop):
+async def test_get(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     assert conn._get(1) is None
 
@@ -265,7 +261,7 @@ def test_get(loop):
     conn.close()
 
 
-def test_get_expired(loop):
+async def test_get_expired(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     key = ConnectionKey('localhost', 80, False, None, None, None, None)
     assert conn._get(key) is None
@@ -277,20 +273,21 @@ def test_get_expired(loop):
     conn.close()
 
 
-def test_get_expired_ssl(loop):
+async def test_get_expired_ssl(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop, enable_cleanup_closed=True)
     key = ConnectionKey('localhost', 80, True, None, None, None, None)
     assert conn._get(key) is None
 
     proto = mock.Mock()
+    transport = proto.transport
     conn._conns[key] = [(proto, loop.time() - 1000)]
     assert conn._get(key) is None
     assert not conn._conns
-    assert conn._cleanup_closed_transports == [proto.close.return_value]
+    assert conn._cleanup_closed_transports == [transport]
     conn.close()
 
 
-def test_release_acquired(loop, key):
+async def test_release_acquired(loop, key) -> None:
     proto = mock.Mock()
     conn = aiohttp.BaseConnector(loop=loop, limit=5)
     conn._release_waiter = mock.Mock()
@@ -309,7 +306,7 @@ def test_release_acquired(loop, key):
     conn.close()
 
 
-def test_release_acquired_closed(loop, key):
+async def test_release_acquired_closed(loop, key) -> None:
     proto = mock.Mock()
     conn = aiohttp.BaseConnector(loop=loop, limit=5)
     conn._release_waiter = mock.Mock()
@@ -324,7 +321,7 @@ def test_release_acquired_closed(loop, key):
     conn.close()
 
 
-def test_release(loop, key):
+async def test_release(loop, key) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     conn._release_waiter = mock.Mock()
 
@@ -341,20 +338,21 @@ def test_release(loop, key):
     conn.close()
 
 
-def test_release_ssl_transport(loop, ssl_key):
+async def test_release_ssl_transport(loop, ssl_key) -> None:
     conn = aiohttp.BaseConnector(loop=loop, enable_cleanup_closed=True)
     conn._release_waiter = mock.Mock()
 
     proto = mock.Mock()
+    transport = proto.transport
     conn._acquired.add(proto)
     conn._acquired_per_host[ssl_key].add(proto)
 
     conn._release(ssl_key, proto, should_close=True)
-    assert conn._cleanup_closed_transports == [proto.close.return_value]
+    assert conn._cleanup_closed_transports == [transport]
     conn.close()
 
 
-def test_release_already_closed(loop):
+async def test_release_already_closed(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
 
     proto = mock.Mock()
@@ -370,7 +368,7 @@ def test_release_already_closed(loop):
     assert not conn._release_acquired.called
 
 
-def test_release_waiter_no_limit(loop, key, key2):
+async def test_release_waiter_no_limit(loop, key, key2) -> None:
     # limit is 0
     conn = aiohttp.BaseConnector(limit=0, loop=loop)
     w = mock.Mock()
@@ -382,7 +380,7 @@ def test_release_waiter_no_limit(loop, key, key2):
     conn.close()
 
 
-def test_release_waiter_first_available(loop, key, key2):
+async def test_release_waiter_first_available(loop, key, key2) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     w1, w2 = mock.Mock(), mock.Mock()
     w1.done.return_value = False
@@ -395,7 +393,7 @@ def test_release_waiter_first_available(loop, key, key2):
     conn.close()
 
 
-def test_release_waiter_release_first(loop, key, key2):
+async def test_release_waiter_release_first(loop, key, key2) -> None:
     conn = aiohttp.BaseConnector(loop=loop, limit=1)
     w1, w2 = mock.Mock(), mock.Mock()
     w1.done.return_value = False
@@ -407,7 +405,7 @@ def test_release_waiter_release_first(loop, key, key2):
     conn.close()
 
 
-def test_release_waiter_skip_done_waiter(loop, key, key2):
+async def test_release_waiter_skip_done_waiter(loop, key, key2) -> None:
     conn = aiohttp.BaseConnector(loop=loop, limit=1)
     w1, w2 = mock.Mock(), mock.Mock()
     w1.done.return_value = True
@@ -419,7 +417,7 @@ def test_release_waiter_skip_done_waiter(loop, key, key2):
     conn.close()
 
 
-def test_release_waiter_per_host(loop, key, key2):
+async def test_release_waiter_per_host(loop, key, key2) -> None:
     # no limit
     conn = aiohttp.BaseConnector(loop=loop, limit=0, limit_per_host=2)
     w1, w2 = mock.Mock(), mock.Mock()
@@ -433,7 +431,7 @@ def test_release_waiter_per_host(loop, key, key2):
     conn.close()
 
 
-def test_release_waiter_no_available(loop, key, key2):
+async def test_release_waiter_no_available(loop, key, key2) -> None:
     # limit is 0
     conn = aiohttp.BaseConnector(limit=0, loop=loop)
     w = mock.Mock()
@@ -446,7 +444,7 @@ def test_release_waiter_no_available(loop, key, key2):
     conn.close()
 
 
-def test_release_close(loop, key):
+async def test_release_close(loop, key) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     proto = mock.Mock(should_close=True)
 
@@ -456,20 +454,20 @@ def test_release_close(loop, key):
     assert proto.close.called
 
 
-def test__drop_acquire_per_host1(loop):
+async def test__drop_acquire_per_host1(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     conn._drop_acquired_per_host(123, 456)
     assert len(conn._acquired_per_host) == 0
 
 
-def test__drop_acquire_per_host2(loop):
+async def test__drop_acquire_per_host2(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     conn._acquired_per_host[123].add(456)
     conn._drop_acquired_per_host(123, 456)
     assert len(conn._acquired_per_host) == 0
 
 
-def test__drop_acquire_per_host3(loop):
+async def test__drop_acquire_per_host3(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     conn._acquired_per_host[123].add(456)
     conn._acquired_per_host[123].add(789)
@@ -478,7 +476,7 @@ def test__drop_acquire_per_host3(loop):
     assert conn._acquired_per_host[123] == {789}
 
 
-async def test_tcp_connector_certificate_error(loop):
+async def test_tcp_connector_certificate_error(loop) -> None:
     req = ClientRequest('GET', URL('https://127.0.0.1:443'), loop=loop)
 
     async def certificate_error(*args, **kwargs):
@@ -493,11 +491,9 @@ async def test_tcp_connector_certificate_error(loop):
     assert isinstance(ctx.value, ssl.CertificateError)
     assert isinstance(ctx.value.certificate_error, ssl.CertificateError)
     assert isinstance(ctx.value, aiohttp.ClientSSLError)
-    assert str(ctx.value) == ('Cannot connect to host 127.0.0.1:443 ssl:True '
-                              '[CertificateError: ()]')
 
 
-async def test_tcp_connector_multiple_hosts_errors(loop):
+async def test_tcp_connector_multiple_hosts_errors(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop)
 
     ip1 = '192.168.1.1'
@@ -602,7 +598,7 @@ async def test_tcp_connector_multiple_hosts_errors(loop):
     assert connected
 
 
-async def test_tcp_connector_resolve_host(loop):
+async def test_tcp_connector_resolve_host(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop, use_dns_cache=True)
 
     res = await conn._resolve_host('localhost', 8080)
@@ -630,7 +626,7 @@ def dns_response(loop):
     return coro
 
 
-async def test_tcp_connector_dns_cache_not_expired(loop, dns_response):
+async def test_tcp_connector_dns_cache_not_expired(loop, dns_response) -> None:
     with mock.patch('aiohttp.connector.DefaultResolver') as m_resolver:
         conn = aiohttp.TCPConnector(
             loop=loop,
@@ -647,7 +643,7 @@ async def test_tcp_connector_dns_cache_not_expired(loop, dns_response):
         )
 
 
-async def test_tcp_connector_dns_cache_forever(loop, dns_response):
+async def test_tcp_connector_dns_cache_forever(loop, dns_response) -> None:
     with mock.patch('aiohttp.connector.DefaultResolver') as m_resolver:
         conn = aiohttp.TCPConnector(
             loop=loop,
@@ -664,7 +660,8 @@ async def test_tcp_connector_dns_cache_forever(loop, dns_response):
         )
 
 
-async def test_tcp_connector_use_dns_cache_disabled(loop, dns_response):
+async def test_tcp_connector_use_dns_cache_disabled(loop,
+                                                    dns_response) -> None:
     with mock.patch('aiohttp.connector.DefaultResolver') as m_resolver:
         conn = aiohttp.TCPConnector(loop=loop, use_dns_cache=False)
         m_resolver().resolve.side_effect = [dns_response(), dns_response()]
@@ -676,7 +673,7 @@ async def test_tcp_connector_use_dns_cache_disabled(loop, dns_response):
         ])
 
 
-async def test_tcp_connector_dns_throttle_requests(loop, dns_response):
+async def test_tcp_connector_dns_throttle_requests(loop, dns_response) -> None:
     with mock.patch('aiohttp.connector.DefaultResolver') as m_resolver:
         conn = aiohttp.TCPConnector(
             loop=loop,
@@ -694,7 +691,8 @@ async def test_tcp_connector_dns_throttle_requests(loop, dns_response):
         )
 
 
-async def test_tcp_connector_dns_throttle_requests_exception_spread(loop):
+async def test_tcp_connector_dns_throttle_requests_exception_spread(
+        loop) -> None:
     with mock.patch('aiohttp.connector.DefaultResolver') as m_resolver:
         conn = aiohttp.TCPConnector(
             loop=loop,
@@ -731,7 +729,7 @@ async def test_tcp_connector_dns_throttle_requests_cancelled_when_close(
             await f
 
 
-async def test_tcp_connector_dns_tracing(loop, dns_response):
+async def test_tcp_connector_dns_tracing(loop, dns_response) -> None:
     session = mock.Mock()
     trace_config_ctx = mock.Mock()
     on_dns_resolvehost_start = mock.Mock(
@@ -806,7 +804,8 @@ async def test_tcp_connector_dns_tracing(loop, dns_response):
         )
 
 
-async def test_tcp_connector_dns_tracing_cache_disabled(loop, dns_response):
+async def test_tcp_connector_dns_tracing_cache_disabled(loop,
+                                                        dns_response) -> None:
     session = mock.Mock()
     trace_config_ctx = mock.Mock()
     on_dns_resolvehost_start = mock.Mock(
@@ -879,7 +878,8 @@ async def test_tcp_connector_dns_tracing_cache_disabled(loop, dns_response):
         ])
 
 
-async def test_tcp_connector_dns_tracing_throttle_requests(loop, dns_response):
+async def test_tcp_connector_dns_tracing_throttle_requests(
+        loop, dns_response) -> None:
     session = mock.Mock()
     trace_config_ctx = mock.Mock()
     on_dns_cache_hit = mock.Mock(
@@ -925,7 +925,7 @@ async def test_tcp_connector_dns_tracing_throttle_requests(loop, dns_response):
         )
 
 
-def test_dns_error(loop):
+async def test_dns_error(loop) -> None:
     connector = aiohttp.TCPConnector(loop=loop)
     connector._resolve_host = make_mocked_coro(
         raise_exception=OSError('dont take it serious'))
@@ -935,10 +935,10 @@ def test_dns_error(loop):
         loop=loop)
 
     with pytest.raises(aiohttp.ClientConnectorError):
-        loop.run_until_complete(connector.connect(req, [], ClientTimeout()))
+        await connector.connect(req, [], ClientTimeout())
 
 
-def test_get_pop_empty_conns(loop):
+async def test_get_pop_empty_conns(loop) -> None:
     # see issue #473
     conn = aiohttp.BaseConnector(loop=loop)
     key = ('127.0.0.1', 80, False)
@@ -948,7 +948,7 @@ def test_get_pop_empty_conns(loop):
     assert not conn._conns
 
 
-def test_release_close_do_not_add_to_pool(loop, key):
+async def test_release_close_do_not_add_to_pool(loop, key) -> None:
     # see issue #473
     conn = aiohttp.BaseConnector(loop=loop)
 
@@ -959,10 +959,10 @@ def test_release_close_do_not_add_to_pool(loop, key):
     assert not conn._conns
 
 
-def test_release_close_do_not_delete_existing_connections(loop, key):
+async def test_release_close_do_not_delete_existing_connections(key) -> None:
     proto1 = mock.Mock()
 
-    conn = aiohttp.BaseConnector(loop=loop)
+    conn = aiohttp.BaseConnector()
     conn._conns[key] = [(proto1, 1)]
 
     proto = mock.Mock(should_close=True)
@@ -973,7 +973,7 @@ def test_release_close_do_not_delete_existing_connections(loop, key):
     conn.close()
 
 
-def test_release_not_started(loop):
+async def test_release_not_started(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     proto = mock.Mock(should_close=False)
     key = 1
@@ -987,7 +987,7 @@ def test_release_not_started(loop):
     conn.close()
 
 
-def test_release_not_opened(loop, key):
+async def test_release_not_opened(loop, key) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
 
     proto = mock.Mock()
@@ -996,7 +996,7 @@ def test_release_not_opened(loop, key):
     assert proto.close.called
 
 
-async def test_connect(loop, key):
+async def test_connect(loop, key) -> None:
     proto = mock.Mock()
     proto.is_connected.return_value = True
 
@@ -1016,7 +1016,7 @@ async def test_connect(loop, key):
     connection.close()
 
 
-async def test_connect_tracing(loop):
+async def test_connect_tracing(loop) -> None:
     session = mock.Mock()
     trace_config_ctx = mock.Mock()
     on_connection_create_start = mock.Mock(
@@ -1065,7 +1065,7 @@ async def test_connect_tracing(loop):
     )
 
 
-async def test_close_during_connect(loop):
+async def test_close_during_connect(loop) -> None:
     proto = mock.Mock()
     proto.is_connected.return_value = True
 
@@ -1087,7 +1087,7 @@ async def test_close_during_connect(loop):
     assert proto.close.called
 
 
-def test_ctor_cleanup():
+async def test_ctor_cleanup() -> None:
     loop = mock.Mock()
     loop.time.return_value = 1.5
     conn = aiohttp.BaseConnector(
@@ -1096,7 +1096,7 @@ def test_ctor_cleanup():
     assert conn._cleanup_closed_handle is not None
 
 
-def test_cleanup(key):
+async def test_cleanup(key) -> None:
     testset = {
         key: [(mock.Mock(), 10),
               (mock.Mock(), 300)],
@@ -1116,8 +1116,9 @@ def test_cleanup(key):
     assert conn._cleanup_handle is not None
 
 
-def test_cleanup_close_ssl_transport(ssl_key):
+async def test_cleanup_close_ssl_transport(ssl_key) -> None:
     proto = mock.Mock()
+    transport = proto.transport
     testset = {ssl_key: [(proto, 10)]}
 
     loop = mock.Mock()
@@ -1129,10 +1130,10 @@ def test_cleanup_close_ssl_transport(ssl_key):
     conn._cleanup()
     assert existing_handle.cancel.called
     assert conn._conns == {}
-    assert conn._cleanup_closed_transports == [proto.close.return_value]
+    assert conn._cleanup_closed_transports == [transport]
 
 
-def test_cleanup2():
+async def test_cleanup2() -> None:
     testset = {1: [(mock.Mock(), 300)]}
     testset[1][0][0].is_connected.return_value = True
 
@@ -1149,7 +1150,7 @@ def test_cleanup2():
     conn.close()
 
 
-def test_cleanup3(key):
+async def test_cleanup3(key) -> None:
     testset = {key: [(mock.Mock(), 290.1),
                      (mock.Mock(), 305.1)]}
     testset[key][0][0].is_connected.return_value = True
@@ -1168,7 +1169,7 @@ def test_cleanup3(key):
     conn.close()
 
 
-def test_cleanup_closed(loop, mocker):
+async def test_cleanup_closed(loop, mocker) -> None:
     if not hasattr(loop, '__dict__'):
         pytest.skip("can not override loop attributes")
 
@@ -1185,7 +1186,7 @@ def test_cleanup_closed(loop, mocker):
     assert cleanup_closed_handle.cancel.called
 
 
-def test_cleanup_closed_disabled(loop, mocker):
+async def test_cleanup_closed_disabled(loop, mocker) -> None:
     conn = aiohttp.BaseConnector(
         loop=loop, enable_cleanup_closed=False)
 
@@ -1196,7 +1197,7 @@ def test_cleanup_closed_disabled(loop, mocker):
     assert not conn._cleanup_closed_transports
 
 
-def test_tcp_connector_ctor(loop):
+async def test_tcp_connector_ctor(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop)
     assert conn._ssl is None
 
@@ -1204,27 +1205,27 @@ def test_tcp_connector_ctor(loop):
     assert conn.family == 0
 
 
-def test_tcp_connector_ctor_fingerprint_valid(loop):
+async def test_tcp_connector_ctor_fingerprint_valid(loop) -> None:
     valid = aiohttp.Fingerprint(hashlib.sha256(b"foo").digest())
     conn = aiohttp.TCPConnector(ssl=valid, loop=loop)
     assert conn._ssl is valid
 
 
-def test_insecure_fingerprint_md5(loop):
+async def test_insecure_fingerprint_md5(loop) -> None:
     with pytest.raises(ValueError):
         aiohttp.TCPConnector(
             ssl=aiohttp.Fingerprint(hashlib.md5(b"foo").digest()),
             loop=loop)
 
 
-def test_insecure_fingerprint_sha1(loop):
+async def test_insecure_fingerprint_sha1(loop) -> None:
     with pytest.raises(ValueError):
         aiohttp.TCPConnector(
             ssl=aiohttp.Fingerprint(hashlib.sha1(b"foo").digest()),
             loop=loop)
 
 
-def test_tcp_connector_clear_dns_cache(loop):
+async def test_tcp_connector_clear_dns_cache(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop)
     hosts = ['a', 'b']
     conn._cached_hosts.add(('localhost', 123), hosts)
@@ -1245,32 +1246,32 @@ def test_tcp_connector_clear_dns_cache(loop):
         conn._cached_hosts.next_addrs(('localhost', 124))
 
 
-def test_tcp_connector_clear_dns_cache_bad_args(loop):
+async def test_tcp_connector_clear_dns_cache_bad_args(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop)
     with pytest.raises(ValueError):
         conn.clear_dns_cache('localhost')
 
 
-def test_dont_recreate_ssl_context(loop):
+async def test_dont_recreate_ssl_context(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop)
     ctx = conn._make_ssl_context(True)
     assert ctx is conn._make_ssl_context(True)
 
 
-def test_dont_recreate_ssl_context2(loop):
+async def test_dont_recreate_ssl_context2(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop)
     ctx = conn._make_ssl_context(False)
     assert ctx is conn._make_ssl_context(False)
 
 
-def test___get_ssl_context1(loop):
+async def test___get_ssl_context1(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop)
     req = mock.Mock()
     req.is_ssl.return_value = False
     assert conn._get_ssl_context(req) is None
 
 
-def test___get_ssl_context2(loop):
+async def test___get_ssl_context2(loop) -> None:
     ctx = ssl.SSLContext()
     conn = aiohttp.TCPConnector(loop=loop)
     req = mock.Mock()
@@ -1279,7 +1280,7 @@ def test___get_ssl_context2(loop):
     assert conn._get_ssl_context(req) is ctx
 
 
-def test___get_ssl_context3(loop):
+async def test___get_ssl_context3(loop) -> None:
     ctx = ssl.SSLContext()
     conn = aiohttp.TCPConnector(loop=loop, ssl=ctx)
     req = mock.Mock()
@@ -1288,7 +1289,7 @@ def test___get_ssl_context3(loop):
     assert conn._get_ssl_context(req) is ctx
 
 
-def test___get_ssl_context4(loop):
+async def test___get_ssl_context4(loop) -> None:
     ctx = ssl.SSLContext()
     conn = aiohttp.TCPConnector(loop=loop, ssl=ctx)
     req = mock.Mock()
@@ -1297,7 +1298,7 @@ def test___get_ssl_context4(loop):
     assert conn._get_ssl_context(req) is conn._make_ssl_context(False)
 
 
-def test___get_ssl_context5(loop):
+async def test___get_ssl_context5(loop) -> None:
     ctx = ssl.SSLContext()
     conn = aiohttp.TCPConnector(loop=loop, ssl=ctx)
     req = mock.Mock()
@@ -1306,7 +1307,7 @@ def test___get_ssl_context5(loop):
     assert conn._get_ssl_context(req) is conn._make_ssl_context(False)
 
 
-def test___get_ssl_context6(loop):
+async def test___get_ssl_context6(loop) -> None:
     conn = aiohttp.TCPConnector(loop=loop)
     req = mock.Mock()
     req.is_ssl.return_value = True
@@ -1314,7 +1315,7 @@ def test___get_ssl_context6(loop):
     assert conn._get_ssl_context(req) is conn._make_ssl_context(True)
 
 
-def test_close_twice(loop):
+async def test_close_twice(loop) -> None:
     proto = mock.Mock()
 
     conn = aiohttp.BaseConnector(loop=loop)
@@ -1330,7 +1331,7 @@ def test_close_twice(loop):
     assert conn.closed
 
 
-def test_close_cancels_cleanup_handle(loop):
+async def test_close_cancels_cleanup_handle(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     conn._release(1, mock.Mock(should_close=False))
     assert conn._cleanup_handle is not None
@@ -1338,7 +1339,7 @@ def test_close_cancels_cleanup_handle(loop):
     assert conn._cleanup_handle is None
 
 
-def test_close_abort_closed_transports(loop):
+async def test_close_abort_closed_transports(loop) -> None:
     tr = mock.Mock()
 
     conn = aiohttp.BaseConnector(loop=loop)
@@ -1350,22 +1351,19 @@ def test_close_abort_closed_transports(loop):
     assert conn.closed
 
 
-def test_close_cancels_cleanup_closed_handle(loop):
+async def test_close_cancels_cleanup_closed_handle(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop, enable_cleanup_closed=True)
     assert conn._cleanup_closed_handle is not None
     conn.close()
     assert conn._cleanup_closed_handle is None
 
 
-def test_ctor_with_default_loop():
-    loop = asyncio.new_event_loop()
-    asyncio.set_event_loop(loop)
+async def test_ctor_with_default_loop(loop) -> None:
     conn = aiohttp.BaseConnector()
     assert loop is conn._loop
-    loop.close()
 
 
-async def test_connect_with_limit(loop, key):
+async def test_connect_with_limit(loop, key) -> None:
     proto = mock.Mock()
     proto.is_connected.return_value = True
 
@@ -1408,7 +1406,7 @@ async def test_connect_with_limit(loop, key):
     conn.close()
 
 
-async def test_connect_queued_operation_tracing(loop, key):
+async def test_connect_queued_operation_tracing(loop, key) -> None:
     session = mock.Mock()
     trace_config_ctx = mock.Mock()
     on_connection_queued_start = mock.Mock(
@@ -1468,7 +1466,7 @@ async def test_connect_queued_operation_tracing(loop, key):
     conn.close()
 
 
-async def test_connect_reuseconn_tracing(loop, key):
+async def test_connect_reuseconn_tracing(loop, key) -> None:
     session = mock.Mock()
     trace_config_ctx = mock.Mock()
     on_connection_reuseconn = mock.Mock(
@@ -1508,7 +1506,7 @@ async def test_connect_reuseconn_tracing(loop, key):
     conn.close()
 
 
-async def test_connect_with_limit_and_limit_per_host(loop, key):
+async def test_connect_with_limit_and_limit_per_host(loop, key) -> None:
     proto = mock.Mock()
     proto.is_connected.return_value = True
 
@@ -1542,7 +1540,7 @@ async def test_connect_with_limit_and_limit_per_host(loop, key):
     conn.close()
 
 
-async def test_connect_with_no_limit_and_limit_per_host(loop, key):
+async def test_connect_with_no_limit_and_limit_per_host(loop, key) -> None:
     proto = mock.Mock()
     proto.is_connected.return_value = True
 
@@ -1574,7 +1572,7 @@ async def test_connect_with_no_limit_and_limit_per_host(loop, key):
     conn.close()
 
 
-async def test_connect_with_no_limits(loop, key):
+async def test_connect_with_no_limits(loop, key) -> None:
     proto = mock.Mock()
     proto.is_connected.return_value = True
 
@@ -1606,7 +1604,7 @@ async def test_connect_with_no_limits(loop, key):
     conn.close()
 
 
-async def test_connect_with_limit_cancelled(loop):
+async def test_connect_with_limit_cancelled(loop) -> None:
 
     proto = mock.Mock()
     proto.is_connected.return_value = True
@@ -1633,7 +1631,7 @@ async def test_connect_with_limit_cancelled(loop):
     connection.close()
 
 
-async def test_connect_with_capacity_release_waiters(loop):
+async def test_connect_with_capacity_release_waiters(loop) -> None:
 
     def check_with_exc(err):
         conn = aiohttp.BaseConnector(limit=1, loop=loop)
@@ -1653,7 +1651,7 @@ async def test_connect_with_capacity_release_waiters(loop):
     check_with_exc(asyncio.TimeoutError())
 
 
-async def test_connect_with_limit_concurrent(loop):
+async def test_connect_with_limit_concurrent(loop) -> None:
     proto = mock.Mock()
     proto.should_close = False
     proto.is_connected.return_value = True
@@ -1714,7 +1712,7 @@ async def test_connect_with_limit_concurrent(loop):
     assert max_connections == num_connections
 
 
-async def test_connect_waiters_cleanup(loop):
+async def test_connect_waiters_cleanup(loop) -> None:
     proto = mock.Mock()
     proto.is_connected.return_value = True
 
@@ -1733,7 +1731,7 @@ async def test_connect_waiters_cleanup(loop):
     assert not conn._waiters.keys()
 
 
-async def test_connect_waiters_cleanup_key_error(loop):
+async def test_connect_waiters_cleanup_key_error(loop) -> None:
     proto = mock.Mock()
     proto.is_connected.return_value = True
 
@@ -1756,7 +1754,7 @@ async def test_connect_waiters_cleanup_key_error(loop):
     assert not conn._waiters.keys() == []
 
 
-async def test_close_with_acquired_connection(loop):
+async def test_close_with_acquired_connection(loop) -> None:
     proto = mock.Mock()
     proto.is_connected.return_value = True
 
@@ -1782,38 +1780,38 @@ async def test_close_with_acquired_connection(loop):
     assert connection.closed
 
 
-def test_default_force_close(loop):
+async def test_default_force_close(loop) -> None:
     connector = aiohttp.BaseConnector(loop=loop)
     assert not connector.force_close
 
 
-def test_limit_property(loop):
+async def test_limit_property(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop, limit=15)
     assert 15 == conn.limit
 
     conn.close()
 
 
-def test_limit_per_host_property(loop):
+async def test_limit_per_host_property(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop, limit_per_host=15)
     assert 15 == conn.limit_per_host
 
     conn.close()
 
 
-def test_limit_property_default(loop):
+async def test_limit_property_default(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     assert conn.limit == 100
     conn.close()
 
 
-def test_limit_per_host_property_default(loop):
+async def test_limit_per_host_property_default(loop) -> None:
     conn = aiohttp.BaseConnector(loop=loop)
     assert conn.limit_per_host == 0
     conn.close()
 
 
-def test_force_close_and_explicit_keep_alive(loop):
+async def test_force_close_and_explicit_keep_alive(loop) -> None:
     with pytest.raises(ValueError):
         aiohttp.BaseConnector(loop=loop, keepalive_timeout=30,
                               force_close=True)
@@ -1827,7 +1825,7 @@ def test_force_close_and_explicit_keep_alive(loop):
     assert conn
 
 
-async def test_error_on_connection(loop, key):
+async def test_error_on_connection(loop, key) -> None:
     conn = aiohttp.BaseConnector(limit=1, loop=loop)
 
     req = mock.Mock()
@@ -1869,7 +1867,7 @@ async def test_error_on_connection(loop, key):
     ret.release()
 
 
-async def test_cancelled_waiter(loop):
+async def test_cancelled_waiter(loop) -> None:
     conn = aiohttp.BaseConnector(limit=1, loop=loop)
     req = mock.Mock()
     req.connection_key = 'key'
@@ -1891,7 +1889,7 @@ async def test_cancelled_waiter(loop):
         await conn2
 
 
-async def test_error_on_connection_with_cancelled_waiter(loop, key):
+async def test_error_on_connection_with_cancelled_waiter(loop, key) -> None:
     conn = aiohttp.BaseConnector(limit=1, loop=loop)
 
     req = mock.Mock()
@@ -1941,7 +1939,7 @@ async def test_error_on_connection_with_cancelled_waiter(loop, key):
     ret.release()
 
 
-async def test_tcp_connector(aiohttp_client, loop):
+async def test_tcp_connector(aiohttp_client, loop) -> None:
 
     async def handler(request):
         return web.Response()
@@ -1956,19 +1954,19 @@ async def test_tcp_connector(aiohttp_client, loop):
 
 @pytest.mark.skipif(not hasattr(socket, 'AF_UNIX'),
                     reason="requires unix socket")
-def test_unix_connector_not_found(loop):
+async def test_unix_connector_not_found(loop) -> None:
     connector = aiohttp.UnixConnector('/' + uuid.uuid4().hex, loop=loop)
 
     req = ClientRequest(
         'GET', URL('http://www.python.org'),
         loop=loop)
     with pytest.raises(aiohttp.ClientConnectorError):
-        loop.run_until_complete(connector.connect(req, None, ClientTimeout()))
+        await connector.connect(req, None, ClientTimeout())
 
 
 @pytest.mark.skipif(not hasattr(socket, 'AF_UNIX'),
                     reason="requires unix socket")
-def test_unix_connector_permission(loop):
+async def test_unix_connector_permission(loop) -> None:
     loop.create_unix_connection = make_mocked_coro(
         raise_exception=PermissionError())
     connector = aiohttp.UnixConnector('/' + uuid.uuid4().hex, loop=loop)
@@ -1977,15 +1975,15 @@ def test_unix_connector_permission(loop):
         'GET', URL('http://www.python.org'),
         loop=loop)
     with pytest.raises(aiohttp.ClientConnectorError):
-        loop.run_until_complete(connector.connect(req, None, ClientTimeout()))
+        await connector.connect(req, None, ClientTimeout())
 
 
-def test_default_use_dns_cache(loop):
-    conn = aiohttp.TCPConnector(loop=loop)
+async def test_default_use_dns_cache() -> None:
+    conn = aiohttp.TCPConnector()
     assert conn.use_dns_cache
 
 
-async def test_resolver_not_called_with_address_is_ip(loop):
+async def test_resolver_not_called_with_address_is_ip(loop) -> None:
     resolver = mock.MagicMock()
     connector = aiohttp.TCPConnector(resolver=resolver)
 
@@ -2000,20 +1998,16 @@ async def test_resolver_not_called_with_address_is_ip(loop):
     resolver.resolve.assert_not_called()
 
 
-async def test_tcp_connector_raise_connector_ssl_error(aiohttp_server):
+async def test_tcp_connector_raise_connector_ssl_error(
+        aiohttp_server, ssl_ctx,
+) -> None:
     async def handler(request):
         return web.Response()
 
     app = web.Application()
     app.router.add_get('/', handler)
 
-    here = os.path.join(os.path.dirname(__file__), '..', 'tests')
-    keyfile = os.path.join(here, 'sample.key')
-    certfile = os.path.join(here, 'sample.crt')
-    sslcontext = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
-    sslcontext.load_cert_chain(certfile, keyfile)
-
-    srv = await aiohttp_server(app, ssl=sslcontext)
+    srv = await aiohttp_server(app, ssl=ssl_ctx)
 
     port = unused_port()
     conn = aiohttp.TCPConnector(local_addr=('127.0.0.1', port))
@@ -2021,37 +2015,40 @@ async def test_tcp_connector_raise_connector_ssl_error(aiohttp_server):
     session = aiohttp.ClientSession(connector=conn)
     url = srv.make_url('/')
 
-    with pytest.raises(aiohttp.ClientConnectorSSLError) as ctx:
-        print(url)
+    if PY_37:
+        err = aiohttp.ClientConnectorCertificateError
+    else:
+        err = aiohttp.ClientConnectorSSLError
+    with pytest.raises(err) as ctx:
         await session.get(url)
 
-    assert isinstance(ctx.value.os_error, ssl.SSLError)
-    assert isinstance(ctx.value, aiohttp.ClientSSLError)
+    if PY_37:
+        assert isinstance(ctx.value, aiohttp.ClientConnectorCertificateError)
+        assert isinstance(ctx.value.certificate_error, ssl.SSLError)
+    else:
+        assert isinstance(ctx.value, aiohttp.ClientSSLError)
+        assert isinstance(ctx.value.os_error, ssl.SSLError)
 
     await session.close()
 
 
-async def test_tcp_connector_do_not_raise_connector_ssl_error(aiohttp_server):
+async def test_tcp_connector_do_not_raise_connector_ssl_error(
+        aiohttp_server, ssl_ctx, client_ssl_ctx,
+) -> None:
     async def handler(request):
         return web.Response()
 
     app = web.Application()
     app.router.add_get('/', handler)
 
-    here = os.path.join(os.path.dirname(__file__), '..', 'tests')
-    keyfile = os.path.join(here, 'sample.key')
-    certfile = os.path.join(here, 'sample.crt')
-    sslcontext = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
-    sslcontext.load_cert_chain(certfile, keyfile)
-
-    srv = await aiohttp_server(app, ssl=sslcontext)
+    srv = await aiohttp_server(app, ssl=ssl_ctx)
     port = unused_port()
     conn = aiohttp.TCPConnector(local_addr=('127.0.0.1', port))
 
     session = aiohttp.ClientSession(connector=conn)
     url = srv.make_url('/')
 
-    r = await session.get(url, ssl=sslcontext)
+    r = await session.get(url, ssl=client_ssl_ctx)
 
     r.release()
     first_conn = next(iter(conn._conns.values()))[0][0]
@@ -2061,14 +2058,14 @@ async def test_tcp_connector_do_not_raise_connector_ssl_error(aiohttp_server):
     except AttributeError:
         _sslcontext = first_conn.transport._sslcontext
 
-    assert _sslcontext is sslcontext
+    assert _sslcontext is client_ssl_ctx
     r.close()
 
     await session.close()
     conn.close()
 
 
-async def test_tcp_connector_uses_provided_local_addr(aiohttp_server):
+async def test_tcp_connector_uses_provided_local_addr(aiohttp_server) -> None:
     async def handler(request):
         return web.Response()
 
@@ -2095,7 +2092,7 @@ async def test_tcp_connector_uses_provided_local_addr(aiohttp_server):
 
 @pytest.mark.skipif(not hasattr(socket, 'AF_UNIX'),
                     reason='requires UNIX sockets')
-async def test_unix_connector(unix_server, unix_sockname):
+async def test_unix_connector(unix_server, unix_sockname) -> None:
     async def handler(request):
         return web.Response()
 
@@ -2121,7 +2118,7 @@ class TestDNSCacheTable:
     def dns_cache_table(self):
         return _DNSCacheTable()
 
-    def test_next_addrs_basic(self, dns_cache_table):
+    def test_next_addrs_basic(self, dns_cache_table) -> None:
         dns_cache_table.add('localhost', ['127.0.0.1'])
         dns_cache_table.add('foo', ['127.0.0.2'])
 
@@ -2132,34 +2129,34 @@ class TestDNSCacheTable:
         with pytest.raises(KeyError):
             dns_cache_table.next_addrs('no-such-host')
 
-    def test_remove(self, dns_cache_table):
+    def test_remove(self, dns_cache_table) -> None:
         dns_cache_table.add('localhost', ['127.0.0.1'])
         dns_cache_table.remove('localhost')
         with pytest.raises(KeyError):
             dns_cache_table.next_addrs('localhost')
 
-    def test_clear(self, dns_cache_table):
+    def test_clear(self, dns_cache_table) -> None:
         dns_cache_table.add('localhost', ['127.0.0.1'])
         dns_cache_table.clear()
         with pytest.raises(KeyError):
             dns_cache_table.next_addrs('localhost')
 
-    def test_not_expired_ttl_None(self, dns_cache_table):
+    def test_not_expired_ttl_None(self, dns_cache_table) -> None:
         dns_cache_table.add('localhost', ['127.0.0.1'])
         assert not dns_cache_table.expired('localhost')
 
-    def test_not_expired_ttl(self):
+    def test_not_expired_ttl(self) -> None:
         dns_cache_table = _DNSCacheTable(ttl=0.1)
         dns_cache_table.add('localhost', ['127.0.0.1'])
         assert not dns_cache_table.expired('localhost')
 
-    async def test_expired_ttl(self, loop):
+    async def test_expired_ttl(self, loop) -> None:
         dns_cache_table = _DNSCacheTable(ttl=0.01)
         dns_cache_table.add('localhost', ['127.0.0.1'])
         await asyncio.sleep(0.02, loop=loop)
         assert dns_cache_table.expired('localhost')
 
-    def test_next_addrs(self, dns_cache_table):
+    def test_next_addrs(self, dns_cache_table) -> None:
         dns_cache_table.add('foo', ['127.0.0.1', '127.0.0.2', '127.0.0.3'])
 
         # Each calls to next_addrs return the hosts using
@@ -2176,7 +2173,7 @@ class TestDNSCacheTable:
         addrs = dns_cache_table.next_addrs('foo')
         assert addrs == ['127.0.0.1', '127.0.0.2', '127.0.0.3']
 
-    def test_next_addrs_single(self, dns_cache_table):
+    def test_next_addrs_single(self, dns_cache_table) -> None:
         dns_cache_table.add('foo', ['127.0.0.1'])
 
         addrs = dns_cache_table.next_addrs('foo')
