@@ -12,10 +12,14 @@ from multidict import CIMultiDictProxy, MultiDict
 from yarl import URL
 
 import aiohttp
-from aiohttp import (FormData, HttpVersion10, HttpVersion11, TraceConfig,
-                     multipart, web)
-from aiohttp.helpers import DEBUG
-
+from aiohttp import (
+    FormData,
+    HttpVersion10,
+    HttpVersion11,
+    TraceConfig,
+    multipart,
+    web,
+)
 
 try:
     import ssl
@@ -30,10 +34,10 @@ def here():
 
 @pytest.fixture
 def fname(here):
-    return here / 'sample.key'
+    return here / 'conftest.py'
 
 
-async def test_simple_get(aiohttp_client):
+async def test_simple_get(aiohttp_client) -> None:
 
     async def handler(request):
         body = await request.read()
@@ -50,7 +54,7 @@ async def test_simple_get(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_simple_get_with_text(aiohttp_client):
+async def test_simple_get_with_text(aiohttp_client) -> None:
 
     async def handler(request):
         body = await request.read()
@@ -67,9 +71,9 @@ async def test_simple_get_with_text(aiohttp_client):
     assert 'OK' == txt
 
 
-@pytest.mark.skipif(not DEBUG,
-                    reason="The check is enabled in debug mode only")
-async def test_handler_returns_not_response(aiohttp_server, aiohttp_client):
+async def test_handler_returns_not_response(aiohttp_server,
+                                            aiohttp_client) -> None:
+    asyncio.get_event_loop().set_debug(True)
     logger = mock.Mock()
 
     async def handler(request):
@@ -80,13 +84,35 @@ async def test_handler_returns_not_response(aiohttp_server, aiohttp_client):
     server = await aiohttp_server(app, logger=logger)
     client = await aiohttp_client(server)
 
-    resp = await client.get('/')
-    assert 500 == resp.status
+    with pytest.raises(aiohttp.ServerDisconnectedError):
+        await client.get('/')
 
-    assert logger.exception.called
+    logger.exception.assert_called_with('Unhandled runtime exception',
+                                        exc_info=mock.ANY)
 
 
-async def test_head_returns_empty_body(aiohttp_client):
+async def test_handler_returns_none(aiohttp_server,
+                                    aiohttp_client) -> None:
+    asyncio.get_event_loop().set_debug(True)
+    logger = mock.Mock()
+
+    async def handler(request):
+        return None
+
+    app = web.Application()
+    app.router.add_get('/', handler)
+    server = await aiohttp_server(app, logger=logger)
+    client = await aiohttp_client(server)
+
+    with pytest.raises(aiohttp.ServerDisconnectedError):
+        await client.get('/')
+
+    # Actual error text is placed in exc_info
+    logger.exception.assert_called_with('Unhandled runtime exception',
+                                        exc_info=mock.ANY)
+
+
+async def test_head_returns_empty_body(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(body=b'test')
@@ -101,7 +127,7 @@ async def test_head_returns_empty_body(aiohttp_client):
     assert '' == txt
 
 
-async def test_response_before_complete(aiohttp_client):
+async def test_response_before_complete(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(body=b'OK')
@@ -118,7 +144,7 @@ async def test_response_before_complete(aiohttp_client):
     assert 'OK' == text
 
 
-async def test_post_form(aiohttp_client):
+async def test_post_form(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -135,7 +161,7 @@ async def test_post_form(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_post_text(aiohttp_client):
+async def test_post_text(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.text()
@@ -154,7 +180,7 @@ async def test_post_text(aiohttp_client):
     assert 'русский' == txt
 
 
-async def test_post_json(aiohttp_client):
+async def test_post_json(aiohttp_client) -> None:
 
     dct = {'key': 'текст'}
 
@@ -179,7 +205,7 @@ async def test_post_json(aiohttp_client):
     assert dct == data
 
 
-async def test_multipart(aiohttp_client):
+async def test_multipart(aiohttp_client) -> None:
     with multipart.MultipartWriter() as writer:
         writer.append('test')
         writer.append_json({'passed': True})
@@ -213,7 +239,7 @@ async def test_multipart(aiohttp_client):
     await resp.release()
 
 
-async def test_multipart_content_transfer_encoding(aiohttp_client):
+async def test_multipart_content_transfer_encoding(aiohttp_client) -> None:
     """For issue #1168"""
     with multipart.MultipartWriter() as writer:
         writer.append(b'\x00' * 10,
@@ -243,7 +269,7 @@ async def test_multipart_content_transfer_encoding(aiohttp_client):
     await resp.release()
 
 
-async def test_render_redirect(aiohttp_client):
+async def test_render_redirect(aiohttp_client) -> None:
 
     async def handler(request):
         raise web.HTTPMovedPermanently(location='/path')
@@ -259,7 +285,7 @@ async def test_render_redirect(aiohttp_client):
     assert '/path' == resp.headers['location']
 
 
-async def test_post_single_file(aiohttp_client):
+async def test_post_single_file(aiohttp_client) -> None:
 
     here = pathlib.Path(__file__).parent
 
@@ -272,7 +298,7 @@ async def test_post_single_file(aiohttp_client):
 
     async def handler(request):
         data = await request.post()
-        assert ['sample.crt'] == list(data.keys())
+        assert ['data.unknown_mime_type'] == list(data.keys())
         for fs in data.values():
             check_file(fs)
             fs.file.close()
@@ -283,13 +309,13 @@ async def test_post_single_file(aiohttp_client):
     app.router.add_post('/', handler)
     client = await aiohttp_client(app)
 
-    fname = here / 'sample.crt'
+    fname = here / 'data.unknown_mime_type'
 
     resp = await client.post('/', data=[fname.open()])
     assert 200 == resp.status
 
 
-async def test_files_upload_with_same_key(aiohttp_client):
+async def test_files_upload_with_same_key(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -322,7 +348,7 @@ async def test_files_upload_with_same_key(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_post_files(aiohttp_client):
+async def test_post_files(aiohttp_client) -> None:
 
     here = pathlib.Path(__file__).parent
 
@@ -335,7 +361,7 @@ async def test_post_files(aiohttp_client):
 
     async def handler(request):
         data = await request.post()
-        assert ['sample.crt', 'sample.key'] == list(data.keys())
+        assert ['data.unknown_mime_type', 'conftest.py'] == list(data.keys())
         for fs in data.values():
             check_file(fs)
             fs.file.close()
@@ -346,13 +372,13 @@ async def test_post_files(aiohttp_client):
     app.router.add_post('/', handler)
     client = await aiohttp_client(app)
 
-    with (here / 'sample.crt').open() as f1:
-        with (here / 'sample.key').open() as f2:
+    with (here / 'data.unknown_mime_type').open() as f1:
+        with (here / 'conftest.py').open() as f2:
             resp = await client.post('/', data=[f1, f2])
             assert 200 == resp.status
 
 
-async def test_release_post_data(aiohttp_client):
+async def test_release_post_data(aiohttp_client) -> None:
 
     async def handler(request):
         await request.release()
@@ -368,7 +394,8 @@ async def test_release_post_data(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_POST_DATA_with_content_transfer_encoding(aiohttp_client):
+async def test_POST_DATA_with_content_transfer_encoding(
+        aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -387,7 +414,7 @@ async def test_POST_DATA_with_content_transfer_encoding(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_post_form_with_duplicate_keys(aiohttp_client):
+async def test_post_form_with_duplicate_keys(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -403,12 +430,12 @@ async def test_post_form_with_duplicate_keys(aiohttp_client):
     assert 200 == resp.status
 
 
-def test_repr_for_application():
+def test_repr_for_application() -> None:
     app = web.Application()
     assert "<Application 0x{:x}>".format(id(app)) == repr(app)
 
 
-async def test_expect_default_handler_unknown(aiohttp_client):
+async def test_expect_default_handler_unknown(aiohttp_client) -> None:
     """Test default Expect handler for unknown Expect value.
 
     A server that does not understand or is unable to comply with any of
@@ -433,7 +460,7 @@ async def test_expect_default_handler_unknown(aiohttp_client):
     assert 417 == resp.status
 
 
-async def test_100_continue(aiohttp_client):
+async def test_100_continue(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -452,7 +479,7 @@ async def test_100_continue(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_100_continue_custom(aiohttp_client):
+async def test_100_continue_custom(aiohttp_client) -> None:
 
     expect_received = False
 
@@ -480,7 +507,7 @@ async def test_100_continue_custom(aiohttp_client):
     assert expect_received
 
 
-async def test_100_continue_custom_response(aiohttp_client):
+async def test_100_continue_custom_response(aiohttp_client) -> None:
 
     async def handler(request):
         data = await request.post()
@@ -511,7 +538,7 @@ async def test_100_continue_custom_response(aiohttp_client):
     assert 403 == resp.status
 
 
-async def test_100_continue_for_not_found(aiohttp_client):
+async def test_100_continue_for_not_found(aiohttp_client) -> None:
 
     app = web.Application()
     client = await aiohttp_client(app)
@@ -520,7 +547,7 @@ async def test_100_continue_for_not_found(aiohttp_client):
     assert 404 == resp.status
 
 
-async def test_100_continue_for_not_allowed(aiohttp_client):
+async def test_100_continue_for_not_allowed(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -533,7 +560,7 @@ async def test_100_continue_for_not_allowed(aiohttp_client):
     assert 405 == resp.status
 
 
-async def test_http11_keep_alive_default(aiohttp_client):
+async def test_http11_keep_alive_default(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -549,7 +576,7 @@ async def test_http11_keep_alive_default(aiohttp_client):
 
 
 @pytest.mark.xfail
-async def test_http10_keep_alive_default(aiohttp_client):
+async def test_http10_keep_alive_default(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -564,7 +591,7 @@ async def test_http10_keep_alive_default(aiohttp_client):
     assert resp.headers['Connection'] == 'keep-alive'
 
 
-async def test_http10_keep_alive_with_headers_close(aiohttp_client):
+async def test_http10_keep_alive_with_headers_close(aiohttp_client) -> None:
 
     async def handler(request):
         await request.read()
@@ -581,7 +608,7 @@ async def test_http10_keep_alive_with_headers_close(aiohttp_client):
     assert 'Connection' not in resp.headers
 
 
-async def test_http10_keep_alive_with_headers(aiohttp_client):
+async def test_http10_keep_alive_with_headers(aiohttp_client) -> None:
 
     async def handler(request):
         await request.read()
@@ -598,7 +625,7 @@ async def test_http10_keep_alive_with_headers(aiohttp_client):
     assert resp.headers['Connection'] == 'keep-alive'
 
 
-async def test_upload_file(aiohttp_client):
+async def test_upload_file(aiohttp_client) -> None:
 
     here = pathlib.Path(__file__).parent
     fname = here / 'aiohttp.png'
@@ -619,7 +646,7 @@ async def test_upload_file(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_upload_file_object(aiohttp_client):
+async def test_upload_file_object(aiohttp_client) -> None:
     here = pathlib.Path(__file__).parent
     fname = here / 'aiohttp.png'
     with fname.open('rb') as f:
@@ -640,7 +667,7 @@ async def test_upload_file_object(aiohttp_client):
         assert 200 == resp.status
 
 
-async def test_empty_content_for_query_without_body(aiohttp_client):
+async def test_empty_content_for_query_without_body(aiohttp_client) -> None:
 
     async def handler(request):
         assert not request.body_exists
@@ -657,7 +684,7 @@ async def test_empty_content_for_query_without_body(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_empty_content_for_query_with_body(aiohttp_client):
+async def test_empty_content_for_query_with_body(aiohttp_client) -> None:
 
     async def handler(request):
         assert request.body_exists
@@ -675,7 +702,7 @@ async def test_empty_content_for_query_with_body(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_get_with_empty_arg(aiohttp_client):
+async def test_get_with_empty_arg(aiohttp_client) -> None:
 
     async def handler(request):
         assert 'arg' in request.query
@@ -690,7 +717,7 @@ async def test_get_with_empty_arg(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_large_header(aiohttp_client):
+async def test_large_header(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -704,7 +731,7 @@ async def test_large_header(aiohttp_client):
     assert 400 == resp.status
 
 
-async def test_large_header_allowed(aiohttp_client, aiohttp_server):
+async def test_large_header_allowed(aiohttp_client, aiohttp_server) -> None:
 
     async def handler(request):
         return web.Response()
@@ -719,7 +746,7 @@ async def test_large_header_allowed(aiohttp_client, aiohttp_server):
     assert 200 == resp.status
 
 
-async def test_get_with_empty_arg_with_equal(aiohttp_client):
+async def test_get_with_empty_arg_with_equal(aiohttp_client) -> None:
 
     async def handler(request):
         assert 'arg' in request.query
@@ -734,7 +761,7 @@ async def test_get_with_empty_arg_with_equal(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_response_with_async_gen(aiohttp_client, fname):
+async def test_response_with_async_gen(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -764,7 +791,7 @@ async def test_response_with_async_gen(aiohttp_client, fname):
     assert resp.headers.get('Content-Length') == str(len(resp_data))
 
 
-async def test_response_with_streamer(aiohttp_client, fname):
+async def test_response_with_streamer(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -795,7 +822,8 @@ async def test_response_with_streamer(aiohttp_client, fname):
     assert resp.headers.get('Content-Length') == str(len(resp_data))
 
 
-async def test_response_with_async_gen_no_params(aiohttp_client, fname):
+async def test_response_with_async_gen_no_params(aiohttp_client,
+                                                 fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -825,7 +853,7 @@ async def test_response_with_async_gen_no_params(aiohttp_client, fname):
     assert resp.headers.get('Content-Length') == str(len(resp_data))
 
 
-async def test_response_with_streamer_no_params(aiohttp_client, fname):
+async def test_response_with_streamer_no_params(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -856,7 +884,7 @@ async def test_response_with_streamer_no_params(aiohttp_client, fname):
     assert resp.headers.get('Content-Length') == str(len(resp_data))
 
 
-async def test_response_with_file(aiohttp_client, fname):
+async def test_response_with_file(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -871,15 +899,20 @@ async def test_response_with_file(aiohttp_client, fname):
     resp = await client.get('/')
     assert 200 == resp.status
     resp_data = await resp.read()
+    expected_content_disposition = (
+        'attachment; filename="conftest.py"; filename*=utf-8\'\'conftest.py'
+    )
     assert resp_data == data
     assert resp.headers.get('Content-Type') in (
-        'application/octet-stream', 'application/pgp-keys')
+        'application/octet-stream', 'text/x-python', 'text/plain',
+    )
     assert resp.headers.get('Content-Length') == str(len(resp_data))
-    assert (resp.headers.get('Content-Disposition') ==
-            'attachment; filename="sample.key"; filename*=utf-8\'\'sample.key')
+    assert (
+        resp.headers.get('Content-Disposition') == expected_content_disposition
+    )
 
 
-async def test_response_with_file_ctype(aiohttp_client, fname):
+async def test_response_with_file_ctype(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -895,14 +928,18 @@ async def test_response_with_file_ctype(aiohttp_client, fname):
     resp = await client.get('/')
     assert 200 == resp.status
     resp_data = await resp.read()
+    expected_content_disposition = (
+        'attachment; filename="conftest.py"; filename*=utf-8\'\'conftest.py'
+    )
     assert resp_data == data
     assert resp.headers.get('Content-Type') == 'text/binary'
     assert resp.headers.get('Content-Length') == str(len(resp_data))
-    assert (resp.headers.get('Content-Disposition') ==
-            'attachment; filename="sample.key"; filename*=utf-8\'\'sample.key')
+    assert (
+        resp.headers.get('Content-Disposition') == expected_content_disposition
+    )
 
 
-async def test_response_with_payload_disp(aiohttp_client, fname):
+async def test_response_with_payload_disp(aiohttp_client, fname) -> None:
 
     with fname.open('rb') as f:
         data = f.read()
@@ -927,7 +964,7 @@ async def test_response_with_payload_disp(aiohttp_client, fname):
             'inline; filename="test.txt"; filename*=utf-8\'\'test.txt')
 
 
-async def test_response_with_payload_stringio(aiohttp_client, fname):
+async def test_response_with_payload_stringio(aiohttp_client, fname) -> None:
 
     async def handler(request):
         return web.Response(body=io.StringIO('test'))
@@ -942,7 +979,7 @@ async def test_response_with_payload_stringio(aiohttp_client, fname):
     assert resp_data == b'test'
 
 
-async def test_response_with_precompressed_body_gzip(aiohttp_client):
+async def test_response_with_precompressed_body_gzip(aiohttp_client) -> None:
 
     async def handler(request):
         headers = {'Content-Encoding': 'gzip'}
@@ -961,7 +998,8 @@ async def test_response_with_precompressed_body_gzip(aiohttp_client):
     assert resp.headers.get('Content-Encoding') == 'gzip'
 
 
-async def test_response_with_precompressed_body_deflate(aiohttp_client):
+async def test_response_with_precompressed_body_deflate(
+        aiohttp_client) -> None:
 
     async def handler(request):
         headers = {'Content-Encoding': 'deflate'}
@@ -980,7 +1018,7 @@ async def test_response_with_precompressed_body_deflate(aiohttp_client):
     assert resp.headers.get('Content-Encoding') == 'deflate'
 
 
-async def test_bad_request_payload(aiohttp_client):
+async def test_bad_request_payload(aiohttp_client) -> None:
 
     async def handler(request):
         assert request.method == 'POST'
@@ -999,7 +1037,7 @@ async def test_bad_request_payload(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_stream_response_multiple_chunks(aiohttp_client):
+async def test_stream_response_multiple_chunks(aiohttp_client) -> None:
 
     async def handler(request):
         resp = web.StreamResponse()
@@ -1020,7 +1058,7 @@ async def test_stream_response_multiple_chunks(aiohttp_client):
     assert b'xyz' == data
 
 
-async def test_start_without_routes(aiohttp_client):
+async def test_start_without_routes(aiohttp_client) -> None:
 
     app = web.Application()
     client = await aiohttp_client(app)
@@ -1029,7 +1067,7 @@ async def test_start_without_routes(aiohttp_client):
     assert 404 == resp.status
 
 
-async def test_requests_count(aiohttp_client):
+async def test_requests_count(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -1052,7 +1090,7 @@ async def test_requests_count(aiohttp_client):
     assert client.server.handler.requests_count == 3
 
 
-async def test_redirect_url(aiohttp_client):
+async def test_redirect_url(aiohttp_client) -> None:
 
     async def redirector(request):
         raise web.HTTPFound(location=URL('/redirected'))
@@ -1069,7 +1107,7 @@ async def test_redirect_url(aiohttp_client):
     assert resp.status == 200
 
 
-async def test_simple_subapp(aiohttp_client):
+async def test_simple_subapp(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(text="OK")
@@ -1086,7 +1124,7 @@ async def test_simple_subapp(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_subapp_reverse_url(aiohttp_client):
+async def test_subapp_reverse_url(aiohttp_client) -> None:
 
     async def handler(request):
         raise web.HTTPMovedPermanently(
@@ -1109,7 +1147,7 @@ async def test_subapp_reverse_url(aiohttp_client):
     assert resp.url.path == '/path/final'
 
 
-async def test_subapp_reverse_variable_url(aiohttp_client):
+async def test_subapp_reverse_variable_url(aiohttp_client) -> None:
 
     async def handler(request):
         raise web.HTTPMovedPermanently(
@@ -1132,7 +1170,7 @@ async def test_subapp_reverse_variable_url(aiohttp_client):
     assert resp.url.path == '/path/final'
 
 
-async def test_subapp_reverse_static_url(aiohttp_client):
+async def test_subapp_reverse_static_url(aiohttp_client) -> None:
     fname = 'aiohttp.png'
 
     async def handler(request):
@@ -1155,7 +1193,7 @@ async def test_subapp_reverse_static_url(aiohttp_client):
         assert body == f.read()
 
 
-async def test_subapp_app(aiohttp_client):
+async def test_subapp_app(aiohttp_client) -> None:
 
     async def handler(request):
         assert request.app is subapp
@@ -1173,7 +1211,7 @@ async def test_subapp_app(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_subapp_not_found(aiohttp_client):
+async def test_subapp_not_found(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(text='OK')
@@ -1188,7 +1226,7 @@ async def test_subapp_not_found(aiohttp_client):
     assert resp.status == 404
 
 
-async def test_subapp_not_found2(aiohttp_client):
+async def test_subapp_not_found2(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(text='OK')
@@ -1203,7 +1241,7 @@ async def test_subapp_not_found2(aiohttp_client):
     assert resp.status == 404
 
 
-async def test_subapp_not_allowed(aiohttp_client):
+async def test_subapp_not_allowed(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(text='OK')
@@ -1219,7 +1257,7 @@ async def test_subapp_not_allowed(aiohttp_client):
     assert resp.headers['Allow'] == 'GET,HEAD'
 
 
-async def test_subapp_cannot_add_app_in_handler(aiohttp_client):
+async def test_subapp_cannot_add_app_in_handler(aiohttp_client) -> None:
 
     async def handler(request):
         request.match_info.add_app(app)
@@ -1235,7 +1273,7 @@ async def test_subapp_cannot_add_app_in_handler(aiohttp_client):
     assert resp.status == 500
 
 
-async def test_subapp_middlewares(aiohttp_client):
+async def test_subapp_middlewares(aiohttp_client) -> None:
     order = []
 
     async def handler(request):
@@ -1266,7 +1304,7 @@ async def test_subapp_middlewares(aiohttp_client):
             (2, subapp2), (2, subapp1), (2, app)] == order
 
 
-async def test_subapp_on_response_prepare(aiohttp_client):
+async def test_subapp_on_response_prepare(aiohttp_client) -> None:
     order = []
 
     async def handler(request):
@@ -1295,7 +1333,7 @@ async def test_subapp_on_response_prepare(aiohttp_client):
     assert [app, subapp1, subapp2] == order
 
 
-async def test_subapp_on_startup(aiohttp_server):
+async def test_subapp_on_startup(aiohttp_server) -> None:
     order = []
 
     async def on_signal(app):
@@ -1315,7 +1353,7 @@ async def test_subapp_on_startup(aiohttp_server):
     assert [app, subapp1, subapp2] == order
 
 
-async def test_subapp_on_shutdown(aiohttp_server):
+async def test_subapp_on_shutdown(aiohttp_server) -> None:
     order = []
 
     async def on_signal(app):
@@ -1336,7 +1374,7 @@ async def test_subapp_on_shutdown(aiohttp_server):
     assert [app, subapp1, subapp2] == order
 
 
-async def test_subapp_on_cleanup(aiohttp_server):
+async def test_subapp_on_cleanup(aiohttp_server) -> None:
     order = []
 
     async def on_signal(app):
@@ -1406,7 +1444,7 @@ async def test_subapp_middleware_context(aiohttp_client,
     assert expected == values
 
 
-async def test_custom_date_header(aiohttp_client):
+async def test_custom_date_header(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response(headers={'Date': 'Sun, 30 Oct 2016 03:13:52 GMT'})
@@ -1420,7 +1458,7 @@ async def test_custom_date_header(aiohttp_client):
     assert resp.headers['Date'] == 'Sun, 30 Oct 2016 03:13:52 GMT'
 
 
-async def test_response_prepared_with_clone(aiohttp_client):
+async def test_response_prepared_with_clone(aiohttp_client) -> None:
 
     async def handler(request):
         cloned = request.clone()
@@ -1436,7 +1474,7 @@ async def test_response_prepared_with_clone(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_app_max_client_size(aiohttp_client):
+async def test_app_max_client_size(aiohttp_client) -> None:
 
     async def handler(request):
         await request.post()
@@ -1455,7 +1493,7 @@ async def test_app_max_client_size(aiohttp_client):
            'actual body size 1048591' in resp_text
 
 
-async def test_app_max_client_size_adjusted(aiohttp_client):
+async def test_app_max_client_size_adjusted(aiohttp_client) -> None:
 
     async def handler(request):
         await request.post()
@@ -1481,7 +1519,7 @@ async def test_app_max_client_size_adjusted(aiohttp_client):
            'actual body size 2097166' in resp_text
 
 
-async def test_app_max_client_size_none(aiohttp_client):
+async def test_app_max_client_size_none(aiohttp_client) -> None:
 
     async def handler(request):
         await request.post()
@@ -1506,7 +1544,7 @@ async def test_app_max_client_size_none(aiohttp_client):
     assert resp_text == 'ok'
 
 
-async def test_post_max_client_size(aiohttp_client):
+async def test_post_max_client_size(aiohttp_client) -> None:
 
     async def handler(request):
         await request.post()
@@ -1525,7 +1563,7 @@ async def test_post_max_client_size(aiohttp_client):
            'actual body size 1024' in resp_text
 
 
-async def test_post_max_client_size_for_file(aiohttp_client):
+async def test_post_max_client_size_for_file(aiohttp_client) -> None:
 
     async def handler(request):
         await request.post()
@@ -1541,7 +1579,7 @@ async def test_post_max_client_size_for_file(aiohttp_client):
     assert 413 == resp.status
 
 
-async def test_response_with_bodypart(aiohttp_client):
+async def test_response_with_bodypart(aiohttp_client) -> None:
 
     async def handler(request):
         reader = await request.multipart()
@@ -1565,7 +1603,7 @@ async def test_response_with_bodypart(aiohttp_client):
                     {'name': 'file', 'filename': 'file', 'filename*': 'file'})
 
 
-async def test_response_with_bodypart_named(aiohttp_client, tmpdir):
+async def test_response_with_bodypart_named(aiohttp_client, tmpdir) -> None:
 
     async def handler(request):
         reader = await request.multipart()
@@ -1593,7 +1631,7 @@ async def test_response_with_bodypart_named(aiohttp_client, tmpdir):
     )
 
 
-async def test_response_with_bodypart_invalid_name(aiohttp_client):
+async def test_response_with_bodypart_invalid_name(aiohttp_client) -> None:
 
     async def handler(request):
         reader = await request.multipart()
@@ -1615,7 +1653,7 @@ async def test_response_with_bodypart_invalid_name(aiohttp_client):
     assert 'content-disposition' not in resp.headers
 
 
-async def test_request_clone(aiohttp_client):
+async def test_request_clone(aiohttp_client) -> None:
 
     async def handler(request):
         r2 = request.clone(method='POST')
@@ -1631,7 +1669,7 @@ async def test_request_clone(aiohttp_client):
     assert 200 == resp.status
 
 
-async def test_await(aiohttp_server):
+async def test_await(aiohttp_server) -> None:
 
     async def handler(request):
         resp = web.StreamResponse(headers={'content-length': str(4)})
@@ -1657,7 +1695,7 @@ async def test_await(aiohttp_server):
         assert resp.connection is None
 
 
-async def test_response_context_manager(aiohttp_server):
+async def test_response_context_manager(aiohttp_server) -> None:
 
     async def handler(request):
         return web.Response()
@@ -1672,7 +1710,7 @@ async def test_response_context_manager(aiohttp_server):
     assert resp.connection is None
 
 
-async def test_response_context_manager_error(aiohttp_server):
+async def test_response_context_manager_error(aiohttp_server) -> None:
 
     async def handler(request):
         return web.Response(text='some text')
@@ -1709,7 +1747,8 @@ async def aiohttp_client_api_context_manager(aiohttp_server):
     assert resp.connection is None
 
 
-async def test_context_manager_close_on_release(aiohttp_server, mocker):
+async def test_context_manager_close_on_release(aiohttp_server,
+                                                mocker) -> None:
 
     async def handler(request):
         resp = web.StreamResponse()
@@ -1734,7 +1773,7 @@ async def test_context_manager_close_on_release(aiohttp_server, mocker):
         assert proto.close.called
 
 
-async def test_iter_any(aiohttp_server):
+async def test_iter_any(aiohttp_server) -> None:
 
     data = b'0123456789' * 1024
 
@@ -1754,7 +1793,7 @@ async def test_iter_any(aiohttp_server):
             assert resp.status == 200
 
 
-async def test_request_tracing(aiohttp_server):
+async def test_request_tracing(aiohttp_server) -> None:
 
     on_request_start = mock.Mock(side_effect=asyncio.coroutine(mock.Mock()))
     on_request_end = mock.Mock(side_effect=asyncio.coroutine(mock.Mock()))
@@ -1829,7 +1868,7 @@ async def test_request_tracing(aiohttp_server):
     await client.close()
 
 
-async def test_return_http_exception_deprecated(aiohttp_client):
+async def test_return_http_exception_deprecated(aiohttp_client) -> None:
 
     async def handler(request):
         return web.HTTPForbidden()
@@ -1842,7 +1881,7 @@ async def test_return_http_exception_deprecated(aiohttp_client):
         await client.get('/')
 
 
-async def test_request_path(aiohttp_client):
+async def test_request_path(aiohttp_client) -> None:
 
     async def handler(request):
         assert request.path_qs == '/path%20to?a=1'
@@ -1860,7 +1899,7 @@ async def test_request_path(aiohttp_client):
     assert 'OK' == txt
 
 
-async def test_app_add_routes(aiohttp_client):
+async def test_app_add_routes(aiohttp_client) -> None:
 
     async def handler(request):
         return web.Response()
@@ -1873,7 +1912,7 @@ async def test_app_add_routes(aiohttp_client):
     assert resp.status == 200
 
 
-async def test_request_headers_type(aiohttp_client):
+async def test_request_headers_type(aiohttp_client) -> None:
 
     async def handler(request):
         assert isinstance(request.headers, CIMultiDictProxy)
@@ -1887,7 +1926,7 @@ async def test_request_headers_type(aiohttp_client):
     assert resp.status == 200
 
 
-async def test_signal_on_error_handler(aiohttp_client):
+async def test_signal_on_error_handler(aiohttp_client) -> None:
 
     async def on_prepare(request, response):
         response.headers['X-Custom'] = 'val'
